@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -41,7 +41,7 @@ EXT unsigned int sa_to_addr(struct sockaddr *, struct host_addr *, u_int16_t *);
 EXT int sa_addr_cmp(struct sockaddr *, struct host_addr *);
 EXT int sa_port_cmp(struct sockaddr *, u_int16_t);
 EXT int host_addr_mask_sa_cmp(struct host_addr *, struct host_mask *, struct sockaddr *);
-EXT unsigned int raw_to_sa(struct sockaddr *, char *, u_int8_t);
+EXT unsigned int raw_to_sa(struct sockaddr *, char *, u_int16_t port, u_int8_t);
 EXT unsigned int sa_to_str(char *, const struct sockaddr *);
 EXT void *pm_htonl6(void *);
 EXT void *pm_ntohl6(void *);
@@ -56,9 +56,12 @@ EXT int is_multicast(struct host_addr *);
 EXT int is_any(struct host_addr *);
 EXT void clean_sin_addr(struct sockaddr *);
 EXT unsigned int label_to_addr(const char *, struct host_addr *, int);
+EXT u_int8_t etype_to_af(u_int16_t);
+EXT u_int16_t af_to_etype(u_int8_t);
 
 #if defined ENABLE_IPV6
 EXT void ipv4_mapped_to_ipv4(struct sockaddr_storage *);
+EXT void ipv4_to_mapped_ipv4(struct sockaddr_storage *);
 #endif
 
 #undef EXT
