@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2015 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
 */
 
 /* 
@@ -98,7 +98,7 @@ assegment_new (u_char type, u_short length)
   
   new = malloc(sizeof (struct assegment));
   if (!new) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (assegment_new: new). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (assegment_new: new). Exiting ..\n", config.name); // XXX
     exit_all(1);
   }
   memset(new, 0, sizeof (struct assegment));
@@ -106,7 +106,7 @@ assegment_new (u_char type, u_short length)
   if (length) {
     new->as = assegment_data_new (length);
     if (!new->as) {
-      Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (assegment_new: new->as). Exiting ..\n", config.name);
+      Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (assegment_new: new->as). Exiting ..\n", config.name); // XXX
       exit_all(1);
     }
     memset(new->as, 0, length);
@@ -313,13 +313,20 @@ assegment_normalise (struct assegment *head)
 }
 
 static struct aspath *
-aspath_new (void)
+aspath_new (struct bgp_peer *peer)
 {
+  struct bgp_misc_structs *bms;
   struct aspath *aspath;
 
+  if (!peer) return NULL;
+
+  bms = bgp_select_misc_db(peer->type);
+
+  if (!bms) return NULL;
+
   aspath = malloc(sizeof (struct aspath));
   if (!aspath) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_new). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/%s ): malloc() failed (aspath_new). Exiting ..\n", config.name, bms->log_str);
     exit_all(1);
   }
   memset (aspath, 0, sizeof (struct aspath));
@@ -340,20 +347,26 @@ aspath_free (struct aspath *aspath)
 
 /* Unintern aspath from AS path bucket. */
 void
-aspath_unintern (struct aspath *aspath)
+aspath_unintern(struct bgp_peer *peer, struct aspath *aspath)
 {
+  struct bgp_rt_structs *inter_domain_routing_db;
   struct aspath *ret;
 
+  if (!peer) return;
+
+  inter_domain_routing_db = bgp_select_routing_db(peer->type);
+
+  if (!inter_domain_routing_db) return;
+
   if (aspath->refcnt)
     aspath->refcnt--;
 
-  if (aspath->refcnt == 0)
-    {
-      /* This aspath must exist in aspath hash table. */
-      ret = hash_release (ashash, aspath);
-      assert (ret != NULL);
-      aspath_free (aspath);
-    }
+  if (aspath->refcnt == 0) {
+    /* This aspath must exist in aspath hash table. */
+    ret = hash_release(inter_domain_routing_db->ashash, aspath);
+    assert (ret != NULL);
+    aspath_free (aspath);
+  }
 }
 
 /* Return the start or end delimiters for a particular Segment type */
@@ -531,7 +544,7 @@ aspath_make_str_count (struct aspath *as)
     {
       str_buf = malloc(1);
       if (!str_buf) {
-	Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_make_str_count). Exiting ..\n", config.name);
+	Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_make_str_count). Exiting ..\n", config.name); // XXX
 	exit_all(1);
       }
       str_buf[0] = '\0';
@@ -554,7 +567,7 @@ aspath_make_str_count (struct aspath *as)
                   ASPATH_STR_DEFAULT_LEN);
   str_buf = malloc(str_size);
   if (!str_buf) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_make_str_count). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_make_str_count). Exiting ..\n", config.name); // XXX
     exit_all(1);
   }
 
@@ -637,15 +650,22 @@ aspath_str_update (struct aspath *as)
 
 /* Intern allocated AS path. */
 struct aspath *
-aspath_intern (struct aspath *aspath)
+aspath_intern (struct bgp_peer *peer, struct aspath *aspath)
 {
+  struct bgp_rt_structs *inter_domain_routing_db;
   struct aspath *find;
+
+  if (!peer) return NULL;
+
+  inter_domain_routing_db = bgp_select_routing_db(peer->type);
+
+  if (!inter_domain_routing_db) return NULL;
   
   /* Assert this AS path structure is not interned. */
   assert (aspath->refcnt == 0);
 
   /* Check AS path hash. */
-  find = hash_get (ashash, aspath, hash_alloc_intern);
+  find = hash_get(peer, inter_domain_routing_db->ashash, aspath, hash_alloc_intern);
 
   if (find != aspath)
     aspath_free (aspath);
@@ -667,7 +687,7 @@ aspath_dup (struct aspath *aspath)
 
   new = malloc(sizeof (struct aspath));
   if (!new) {
-    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_dup). Exiting ..\n", config.name);
+    Log(LOG_ERR, "ERROR ( %s/core/BGP ): malloc() failed (aspath_dup). Exiting ..\n", config.name); // XXX
     exit_all(1);
   }
   memset(new, 0, sizeof(struct aspath));
@@ -720,8 +740,6 @@ assegments_parse(char *s, size_t length, int use32bit)
 
   aspathlen = length;
   
-  // while ((length > AS_HEADER_SIZE) && (bytes < length)) {
-  // while ((aspathlen > AS_HEADER_SIZE) && (bytes < aspathlen)) {
   while (aspathlen > 0) {
       int i;
       int seg_size;
@@ -771,11 +789,18 @@ assegments_parse(char *s, size_t length, int use32bit)
 
 /* AS path parse function. If there is same AS path in the the AS
    path hash then return it else make new AS path structure. */
-struct aspath *aspath_parse(char *s, size_t length, int use32bit)
+struct aspath *aspath_parse(struct bgp_peer *peer, char *s, size_t length, int use32bit)
 {
+  struct bgp_rt_structs *inter_domain_routing_db;
   struct aspath as;
   struct aspath *find;
 
+  if (!peer) return NULL;
+
+  inter_domain_routing_db = bgp_select_routing_db(peer->type);
+
+  if (!inter_domain_routing_db) return NULL;
+
   /* If length is odd it's malformed AS path. */
   /* Nit-picking: if (use32bit == 0) it is malformed if odd,
    * otherwise its malformed when length is larger than 2 and (length-2) 
@@ -788,7 +813,7 @@ struct aspath *aspath_parse(char *s, size_t length, int use32bit)
   as.segments = assegments_parse(s, length, use32bit);
   
   /* If already same aspath exist then return it. */
-  find = hash_get (ashash, &as, aspath_hash_alloc);
+  find = hash_get (peer, inter_domain_routing_db->ashash, &as, aspath_hash_alloc);
   
   /* aspath_hash_alloc dupes segments too. that probably could be
    * optimised out.
@@ -804,132 +829,6 @@ struct aspath *aspath_parse(char *s, size_t length, int use32bit)
   return find;
 }
 
-static struct assegment *
-aspath_aggregate_as_set_add (struct aspath *aspath, struct assegment *asset,
-			     as_t as)
-{
-  int i;
-
-  /* If this is first AS set member, create new as-set segment. */
-  if (asset == NULL)
-    {
-      asset = assegment_new (AS_SET, 1);
-      if (! aspath->segments)
-	aspath->segments = asset;
-      else
-        {
-          struct assegment *seg = aspath->segments;
-          while (seg->next)
-            seg = seg->next;
-          seg->next = asset;
-        }
-      asset->type = AS_SET;
-      asset->length = 1;
-      asset->as[0] = as;
-    }
-  else
-    {
-      /* Check this AS value already exists or not. */
-      for (i = 0; i < asset->length; i++)
-	if (asset->as[i] == as)
-	  return asset;
-      
-      asset->length++;
-      asset->as = realloc(asset->as, asset->length * AS_VALUE_SIZE);
-      asset->as[asset->length - 1] = as;
-    }
-  
-
-  return asset;
-}
-
-/* Modify as1 using as2 for aggregation. */
-struct aspath *
-aspath_aggregate (struct aspath *as1, struct aspath *as2)
-{
-  int i;
-  int minlen;
-  int match;
-  int from;
-  struct assegment *seg1 = as1->segments;
-  struct assegment *seg2 = as2->segments;
-  struct aspath *aspath = NULL;
-  struct assegment *asset;
-  struct assegment *prevseg = NULL;
-
-  match = 0;
-  minlen = 0;
-  aspath = NULL;
-  asset = NULL;
-
-  /* First of all check common leading sequence. */
-  while (seg1 && seg2)
-    {      
-      /* Check segment type. */
-      if (seg1->type != seg2->type)
-	break;
-
-      /* Minimum segment length. */
-      minlen = MIN(seg1->length, seg2->length);
-
-      for (match = 0; match < minlen; match++)
-	if (seg1->as[match] != seg2->as[match])
-	  break;
-
-      if (match)
-	{
-	  struct assegment *seg = assegment_new (seg1->type, 0);
-	  
-	  seg = assegment_append_asns (seg, seg1->as, match);
-
-	  if (! aspath)
-	    {
-	      aspath = aspath_new ();
-	      aspath->segments = seg;
-	     }
-	  else
-	    prevseg->next = seg;
-	  
-	  prevseg = seg;
-	}
-
-      if (match != minlen || match != seg1->length 
-	  || seg1->length != seg2->length)
-	break;
-      
-      seg1 = seg1->next;
-      seg2 = seg2->next;
-    }
-
-  if (! aspath)
-    aspath = aspath_new();
-
-  /* Make as-set using rest of all information. */
-  from = match;
-  while (seg1)
-    {
-      for (i = from; i < seg1->length; i++)
-	asset = aspath_aggregate_as_set_add (aspath, asset, seg1->as[i]);
-      
-      from = 0;
-      seg1 = seg1->next;
-    }
-
-  from = match;
-  while (seg2)
-    {
-      for (i = from; i < seg2->length; i++)
-	asset = aspath_aggregate_as_set_add (aspath, asset, seg2->as[i]);
-
-      from = 0;
-      seg2 = seg2->next;
-    }
-  
-  assegment_normalise (aspath->segments);
-  aspath_str_update (aspath);
-  return aspath;
-}
-
 /* When a BGP router receives an UPDATE with an MP_REACH_NLRI
    attribute, check the leftmost AS number in the AS_PATH attribute is
    or not the peer's AS number. */ 
@@ -1019,203 +918,6 @@ aspath_merge (struct aspath *as1, struct aspath *as2)
   return as2;
 }
 
-/* Prepend as1 to as2.  as2 should be uninterned aspath. */
-struct aspath *
-aspath_prepend (struct aspath *as1, struct aspath *as2)
-{
-  struct assegment *seg1;
-  struct assegment *seg2;
-
-  if (! as1 || ! as2)
-    return NULL;
-  
-  seg1 = as1->segments;
-  seg2 = as2->segments;
-  
-  /* If as2 is empty, only need to dupe as1's chain onto as2 */
-  if (seg2 == NULL)
-    {
-      as2->segments = assegment_dup_all (as1->segments);
-      aspath_str_update (as2);
-      return as2;
-    }
-  
-  /* If as1 is empty AS, no prepending to do. */
-  if (seg1 == NULL)
-    return as2;
-  
-  /* find the tail as1's segment chain. */
-  while (seg1 && seg1->next)
-    seg1 = seg1->next;
-
-  /* Compare last segment type of as1 and first segment type of as2. */
-  if (seg1->type != seg2->type)
-    return aspath_merge (as1, as2);
-
-  if (seg1->type == AS_SEQUENCE)
-    {
-      /* We have two chains of segments, as1->segments and seg2, 
-       * and we have to attach them together, merging the attaching
-       * segments together into one.
-       * 
-       * 1. dupe as1->segments onto head of as2
-       * 2. merge seg2's asns onto last segment of this new chain
-       * 3. attach chain after seg2
-       */
-      
-      /* dupe as1 onto as2's head */
-      seg1 = as2->segments = assegment_dup_all (as1->segments);
-      
-      /* refind the tail of as2, reusing seg1 */
-      while (seg1 && seg1->next)
-        seg1 = seg1->next;
-      
-      /* merge the old head, seg2, into tail, seg1 */
-      seg1 = assegment_append_asns (seg1, seg2->as, seg2->length);
-      
-      /* bypass the merged seg2, and attach any chain after it to
-       * chain descending from as2's head
-       */
-      seg1->next = seg2->next;
-      
-      /* seg2 is now referenceless and useless*/
-      assegment_free (seg2);
-      
-      /* we've now prepended as1's segment chain to as2, merging
-       * the inbetween AS_SEQUENCE of seg2 in the process 
-       */
-      aspath_str_update (as2);
-      return as2;
-    }
-  else
-    {
-      /* AS_SET merge code is needed at here. */
-      return aspath_merge (as1, as2);
-    }
-  /* XXX: Ermmm, what if as1 has multiple segments?? */
-  
-  /* Not reached */
-}
-
-/* Iterate over AS_PATH segments and wipe all occurences of the
- * listed AS numbers. Hence some segments may lose some or even
- * all data on the way, the operation is implemented as a smarter
- * version of aspath_dup(), which allocates memory to hold the new
- * data, not the original. The new AS path is returned.
- */
-struct aspath *
-aspath_filter_exclude (struct aspath * source, struct aspath * exclude_list)
-{
-  struct assegment * srcseg, * exclseg, * lastseg;
-  struct aspath * newpath;
-
-  newpath = aspath_new();
-  lastseg = NULL;
-
-  for (srcseg = source->segments; srcseg; srcseg = srcseg->next)
-  {
-    unsigned i, y, newlen = 0, done = 0, skip_as;
-    struct assegment * newseg;
-
-    /* Find out, how much ASns are we going to pick from this segment.
-     * We can't perform filtering right inline, because the size of
-     * the new segment isn't known at the moment yet.
-     */
-    for (i = 0; i < srcseg->length; i++)
-    {
-      skip_as = 0;
-      for (exclseg = exclude_list->segments; exclseg && !skip_as; exclseg = exclseg->next)
-        for (y = 0; y < exclseg->length; y++)
-          if (srcseg->as[i] == exclseg->as[y])
-          {
-            skip_as = 1;
-            // There's no sense in testing the rest of exclusion list, bail out.
-            break;
-          }
-      if (!skip_as)
-        newlen++;
-    }
-    /* newlen is now the number of ASns to copy */
-    if (!newlen)
-      continue;
-
-    /* Actual copying. Allocate memory and iterate once more, performing filtering. */
-    newseg = assegment_new (srcseg->type, newlen);
-    for (i = 0; i < srcseg->length; i++)
-    {
-      skip_as = 0;
-      for (exclseg = exclude_list->segments; exclseg && !skip_as; exclseg = exclseg->next)
-        for (y = 0; y < exclseg->length; y++)
-          if (srcseg->as[i] == exclseg->as[y])
-          {
-            skip_as = 1;
-            break;
-          }
-      if (skip_as)
-        continue;
-      newseg->as[done++] = srcseg->as[i];
-    }
-    /* At his point newlen must be equal to done, and both must be positive. Append
-     * the filtered segment to the gross result. */
-    if (!lastseg)
-      newpath->segments = newseg;
-    else
-      lastseg->next = newseg;
-    lastseg = newseg;
-  }
-  aspath_str_update (newpath);
-  /* We are happy returning even an empty AS_PATH, because the administrator
-   * might expect this very behaviour. There's a mean to avoid this, if necessary,
-   * by having a match rule against certain AS_PATH regexps in the route-map index.
-   */
-  aspath_free (source);
-  return newpath;
-}
-
-/* Add specified AS to the leftmost of aspath. */
-static struct aspath *
-aspath_add_one_as (struct aspath *aspath, as_t asno, u_char type)
-{
-  struct assegment *assegment = aspath->segments;
-
-  /* In case of empty aspath. */
-  if (assegment == NULL || assegment->length == 0)
-    {
-      aspath->segments = assegment_new (type, 1);
-      aspath->segments->as[0] = asno;
-      
-      if (assegment)
-	assegment_free (assegment);
-
-      return aspath;
-    }
-
-  if (assegment->type == type)
-    aspath->segments = assegment_prepend_asns (aspath->segments, asno, 1);
-  else 
-    {
-      /* create new segment
-       * push it onto head of aspath's segment chain 
-       */
-      struct assegment *newsegment;
-      
-      newsegment = assegment_new (type, 1);
-      newsegment->as[0] = asno;
-      
-      newsegment->next = assegment;
-      aspath->segments = newsegment;
-    }
-
-  return aspath;
-}
-
-/* Add specified AS to the leftmost of aspath. */
-struct aspath *
-aspath_add_seq (struct aspath *aspath, as_t asno)
-{
-  return aspath_add_one_as (aspath, asno, AS_SEQUENCE);
-}
-
 /* Compare leftmost AS value for MED check.  If as1's leftmost AS and
    as2's leftmost AS is same return 1. */
 int
@@ -1257,14 +959,13 @@ aspath_cmp_left (const struct aspath *aspath1, const struct aspath *aspath2)
  * interned by the caller, as desired.
  */
 struct aspath *
-aspath_reconcile_as4 ( struct aspath *aspath, struct aspath *as4path)
+aspath_reconcile_as4 (struct bgp_peer *peer, struct aspath *aspath, struct aspath *as4path)
 {
   struct assegment *seg, *newseg, *prevseg = NULL;
   struct aspath *newpath = NULL, *mergedpath;
   int hops, cpasns = 0;
   
-  if (!aspath)
-    return NULL;
+  if (!aspath || !peer) return NULL;
   
   seg = aspath->segments;
   
@@ -1317,7 +1018,7 @@ aspath_reconcile_as4 ( struct aspath *aspath, struct aspath *as4path)
 
       if (!newpath)
         {
-          newpath = aspath_new ();
+          newpath = aspath_new (peer);
           newpath->segments = newseg;
         }
       else
@@ -1360,256 +1061,6 @@ aspath_cmp_left_confed (const struct aspath *aspath1, const struct aspath *aspat
 
   return 0;
 }
-
-/* Delete all leading AS_CONFED_SEQUENCE/SET segments from aspath.
- * See RFC3065, 6.1 c1 */
-struct aspath *
-aspath_delete_confed_seq (struct aspath *aspath)
-{
-  struct assegment *seg;
-
-  if (!(aspath && aspath->segments))
-    return aspath;
-
-  seg = aspath->segments;
-  
-  /* "if the first path segment of the AS_PATH is 
-   *  of type AS_CONFED_SEQUENCE,"
-   */
-  if (aspath->segments->type != AS_CONFED_SEQUENCE)
-    return aspath;
-
-  /* "... that segment and any immediately following segments 
-   *  of the type AS_CONFED_SET or AS_CONFED_SEQUENCE are removed 
-   *  from the AS_PATH attribute,"
-   */
-  while (seg && 
-         (seg->type == AS_CONFED_SEQUENCE || seg->type == AS_CONFED_SET))
-    {
-      aspath->segments = seg->next;
-      assegment_free (seg);
-      seg = aspath->segments;
-    }
-  aspath_str_update (aspath);
-  return aspath;
-}
-
-/* Add new AS number to the leftmost part of the aspath as
-   AS_CONFED_SEQUENCE.  */
-struct aspath*
-aspath_add_confed_seq (struct aspath *aspath, as_t asno)
-{
-  return aspath_add_one_as (aspath, asno, AS_CONFED_SEQUENCE);
-}
-
-/* Add new as value to as path structure. */
-static void
-aspath_as_add (struct aspath *as, as_t asno)
-{
-  struct assegment *seg = as->segments;
-
-  if (!seg)
-    return;
-  
-  /* Last segment search procedure. */
-  while (seg->next)
-    seg = seg->next;
-
-  assegment_append_asns (seg, &asno, 1);
-}
-
-/* Add new as segment to the as path. */
-static void
-aspath_segment_add (struct aspath *as, int type)
-{
-  struct assegment *seg = as->segments;
-  struct assegment *new = assegment_new (type, 0);
-
-  if (seg)
-    {
-      while (seg->next)
-	seg = seg->next;
-      seg->next = new;
-    }
-  else
-    as->segments = new;
-}
-
-struct aspath *
-aspath_empty (void)
-{
-  return aspath_parse (NULL, 0, 1); /* 32Bit ;-) */
-}
-
-struct aspath *
-aspath_empty_get (void)
-{
-  struct aspath *aspath;
-
-  aspath = aspath_new ();
-  aspath->str = aspath_make_str_count (aspath);
-  return aspath;
-}
-
-unsigned long
-aspath_count (void)
-{
-  return ashash->count;
-}     
-
-/* 
-   Theoretically, one as path can have:
-
-   One BGP packet size should be less than 4096.
-   One BGP attribute size should be less than 4096 - BGP header size.
-   One BGP aspath size should be less than 4096 - BGP header size -
-       BGP mandantry attribute size.
-*/
-
-/* AS path string lexical token enum. */
-enum as_token
-{
-  as_token_asval,
-  as_token_set_start,
-  as_token_set_end,
-  as_token_confed_seq_start,
-  as_token_confed_seq_end,
-  as_token_confed_set_start,
-  as_token_confed_set_end,
-  as_token_unknown
-};
-
-/* Return next token and point for string parse. */
-static const char *
-aspath_gettoken (const char *buf, enum as_token *token, u_long *asno)
-{
-  const char *p = buf;
-
-  /* Skip seperators (space for sequences, ',' for sets). */
-  while (isspace ((int) *p) || *p == ',')
-    p++;
-
-  /* Check the end of the string and type specify characters
-     (e.g. {}()). */
-  switch (*p)
-    {
-    case '\0':
-      return NULL;
-    case '{':
-      *token = as_token_set_start;
-      p++;
-      return p;
-    case '}':
-      *token = as_token_set_end;
-      p++;
-      return p;
-    case '(':
-      *token = as_token_confed_seq_start;
-      p++;
-      return p;
-    case ')':
-      *token = as_token_confed_seq_end;
-      p++;
-      return p;
-    case '[':
-      *token = as_token_confed_set_start;
-      p++;
-      return p;
-    case ']':
-      *token = as_token_confed_set_end;
-      p++;
-      return p;
-    }
-
-  /* Check actual AS value. */
-  if (isdigit ((int) *p)) 
-    {
-      u_short asval;
-      
-      *token = as_token_asval;
-      asval = (*p - '0');
-      p++;
-      
-      while (isdigit ((int) *p)) 
-        {
-          asval *= 10;
-          asval += (*p - '0');
-          p++;
-        }
-      *asno = asval;
-      return p;
-    }
-  
-  /* There is no match then return unknown token. */
-  *token = as_token_unknown;
-  return  p++;
-}
-
-struct aspath *
-aspath_str2aspath (const char *str)
-{
-  enum as_token token = as_token_unknown;
-  u_short as_type;
-  u_long asno = 0;
-  struct aspath *aspath;
-  int needtype;
-
-  aspath = aspath_new ();
-
-  /* We start default type as AS_SEQUENCE. */
-  as_type = AS_SEQUENCE;
-  needtype = 1;
-
-  while ((str = aspath_gettoken (str, &token, &asno)) != NULL)
-    {
-      switch (token)
-	{
-	case as_token_asval:
-	  if (needtype)
-	    {
-	      aspath_segment_add (aspath, as_type);
-	      needtype = 0;
-	    }
-	  aspath_as_add (aspath, asno);
-	  break;
-	case as_token_set_start:
-	  as_type = AS_SET;
-	  aspath_segment_add (aspath, as_type);
-	  needtype = 0;
-	  break;
-	case as_token_set_end:
-	  as_type = AS_SEQUENCE;
-	  needtype = 1;
-	  break;
-	case as_token_confed_seq_start:
-	  as_type = AS_CONFED_SEQUENCE;
-	  aspath_segment_add (aspath, as_type);
-	  needtype = 0;
-	  break;
-	case as_token_confed_seq_end:
-	  as_type = AS_SEQUENCE;
-	  needtype = 1;
-	  break;
-	case as_token_confed_set_start:
-	  as_type = AS_CONFED_SET;
-	  aspath_segment_add (aspath, as_type);
-	  needtype = 0;
-	  break;
-	case as_token_confed_set_end:
-	  as_type = AS_SEQUENCE;
-	  needtype = 1;
-	  break;
-	case as_token_unknown:
-	default:
-	  aspath_free (aspath);
-	  return NULL;
-	}
-    }
-
-  aspath->str = aspath_make_str_count (aspath);
-
-  return aspath;
-}
 
 /* Make hash value by raw aspath data. */
 unsigned int
@@ -1653,17 +1104,11 @@ aspath_cmp (const void *arg1, const void *arg2)
 
 /* AS path hash initialize. */
 void
-aspath_init (struct hash **loc_ashash)
+aspath_init (int buckets, struct hash **loc_ashash)
 {
   (*loc_ashash) = hash_create_size (32767, aspath_key_make, aspath_cmp);
 }
 
-void
-aspath_finish (void)
-{
-  hash_free (ashash);
-}
-
 /* return and as path value */
 const char *
 aspath_print (struct aspath *as)
