@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -24,6 +24,7 @@
 
 /* includes */
 #include "pmacct.h"
+#include "addr.h"
 #include "bgp.h"
 #include "../bmp/bmp.h"
 #if defined WITH_RABBITMQ
@@ -33,13 +34,14 @@
 #include "kafka_common.h"
 #endif
 
-int bgp_peer_log_msg(struct bgp_node *route, struct bgp_info *ri, safi_t safi, char *event_type, int output, int log_type)
+int bgp_peer_log_msg(struct bgp_node *route, struct bgp_info *ri, afi_t afi, safi_t safi, char *event_type, int output, int log_type)
 {
   struct bgp_misc_structs *bms;
   char log_rk[SRVBUFLEN];
   struct bgp_peer *peer;
   struct bgp_attr *attr;
   int ret = 0, amqp_ret = 0, kafka_ret = 0, etype = BGP_LOGDUMP_ET_NONE;
+  pid_t writer_pid = getpid();
 
   if (!ri || !ri->peer || !ri->peer->log || !event_type) return ERR;
 
@@ -67,7 +69,7 @@ int bgp_peer_log_msg(struct bgp_node *route, struct bgp_info *ri, safi_t safi, c
   if (output == PRINT_OUTPUT_JSON) {
 #ifdef WITH_JANSSON
     char ip_address[INET6_ADDRSTRLEN];
-    json_t *obj = json_object(), *kv;
+    json_t *obj = json_object();
 
     char empty[] = "";
     char prefix_str[INET6_ADDRSTRLEN], nexthop_str[INET6_ADDRSTRLEN];
@@ -75,112 +77,91 @@ int bgp_peer_log_msg(struct bgp_node *route, struct bgp_info *ri, safi_t safi, c
 
     /* no need for seq for "dump" event_type */
     if (etype == BGP_LOGDUMP_ET_LOG) {
-      kv = json_pack("{sI}", "seq", (json_int_t)bms->log_seq);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "seq", json_integer((json_int_t)bms->log_seq));
       bgp_peer_log_seq_increment(&bms->log_seq);
 
       switch (log_type) {
       case BGP_LOG_TYPE_UPDATE:
-	kv = json_pack("{ss}", "log_type", "update");
+	json_object_set_new_nocheck(obj, "log_type", json_string("update"));
 	break;
       case BGP_LOG_TYPE_WITHDRAW:
-	kv = json_pack("{ss}", "log_type", "withdraw");
+	json_object_set_new_nocheck(obj, "log_type", json_string("withdraw"));
 	break;
       case BGP_LOG_TYPE_DELETE:
-	kv = json_pack("{ss}", "log_type", "delete");
+	json_object_set_new_nocheck(obj, "log_type", json_string("delete"));
 	break;
       default:
-	kv = json_pack("{sI}", "log_type", (json_int_t)log_type);
+        json_object_set_new_nocheck(obj, "log_type", json_integer((json_int_t)log_type));
 	break;
       }
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
     }
 
-    if (etype == BGP_LOGDUMP_ET_LOG) {
-      kv = json_pack("{ss}", "timestamp", bms->log_tstamp_str);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
-    }
-    else if (etype == BGP_LOGDUMP_ET_DUMP) {
-      kv = json_pack("{ss}", "timestamp", bms->dump.tstamp_str);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
-    }
+    if (etype == BGP_LOGDUMP_ET_LOG)
+      json_object_set_new_nocheck(obj, "timestamp", json_string(bms->log_tstamp_str));
+    else if (etype == BGP_LOGDUMP_ET_DUMP)
+      json_object_set_new_nocheck(obj, "timestamp", json_string(bms->dump.tstamp_str));
 
     if (bms->bgp_peer_log_msg_extras) bms->bgp_peer_log_msg_extras(peer, output, obj);
 
-    addr_to_str(ip_address, &peer->addr);
-    kv = json_pack("{ss}", bms->peer_str, ip_address);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    if (ri && ri->extra && ri->extra->bmed.id && bms->bgp_peer_logdump_extra_data)
+      bms->bgp_peer_logdump_extra_data(&ri->extra->bmed, output, obj);
 
-    kv = json_pack("{ss}", "event_type", event_type);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    addr_to_str(ip_address, &peer->addr);
+    json_object_set_new_nocheck(obj, bms->peer_str, json_string(ip_address));
+
+    json_object_set_new_nocheck(obj, "event_type", json_string(event_type));
+
+    json_object_set_new_nocheck(obj, "afi", json_integer((json_int_t)afi));
+
+    json_object_set_new_nocheck(obj, "safi", json_integer((json_int_t)safi));
 
     if (route) {
       memset(prefix_str, 0, INET6_ADDRSTRLEN);
       prefix2str(&route->p, prefix_str, INET6_ADDRSTRLEN);
-      kv = json_pack("{ss}", "ip_prefix", prefix_str);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "ip_prefix", json_string(prefix_str));
     }
 
-    if (ri && ri->extra && ri->extra->path_id) {
-      kv = json_pack("{sI}", "as_path_id", (json_int_t)ri->extra->path_id);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
-    }
+    if (ri && ri->extra && ri->extra->path_id)
+      json_object_set_new_nocheck(obj, "as_path_id", json_integer((json_int_t)ri->extra->path_id));
 
     if (attr) {
       memset(nexthop_str, 0, INET6_ADDRSTRLEN);
       if (attr->mp_nexthop.family) addr_to_str(nexthop_str, &attr->mp_nexthop);
       else inet_ntop(AF_INET, &attr->nexthop, nexthop_str, INET6_ADDRSTRLEN);
-      kv = json_pack("{ss}", "bgp_nexthop", nexthop_str);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "bgp_nexthop", json_string(nexthop_str));
 
       aspath = attr->aspath ? attr->aspath->str : empty;
-      kv = json_pack("{ss}", "as_path", aspath);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "as_path", json_string(aspath));
 
-      if (attr->community) {
-        kv = json_pack("{ss}", "comms", attr->community->str);
-        json_object_update_missing(obj, kv);
-        json_decref(kv);
-      }
+      if (attr->community)
+	json_object_set_new_nocheck(obj, "comms", json_string(attr->community->str));
 
-      if (attr->ecommunity) {
-        kv = json_pack("{ss}", "ecomms", attr->ecommunity->str);
-        json_object_update_missing(obj, kv);
-        json_decref(kv);
-      }
+      if (attr->ecommunity)
+	json_object_set_new_nocheck(obj, "ecomms", json_string(attr->ecommunity->str));
 
-      kv = json_pack("{sI}", "origin", (json_int_t)attr->origin);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      if (attr->lcommunity)
+	json_object_set_new_nocheck(obj, "lcomms", json_string(attr->lcommunity->str));
 
-      kv = json_pack("{sI}", "local_pref", (json_int_t)attr->local_pref);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "origin", json_integer((json_int_t)attr->origin));
 
-      if (attr->med) {
-        kv = json_pack("{sI}", "med", (json_int_t)attr->med);
-        json_object_update_missing(obj, kv);
-        json_decref(kv);
-      }
+      json_object_set_new_nocheck(obj, "local_pref", json_integer((json_int_t)attr->local_pref));
+
+      if (attr->med)
+	json_object_set_new_nocheck(obj, "med", json_integer((json_int_t)attr->med));
     }
 
-    if (safi == SAFI_MPLS_VPN) {
-      u_char rd_str[SRVBUFLEN];
+    if (safi == SAFI_MPLS_LABEL || safi == SAFI_MPLS_VPN) {
+      u_char label_str[SHORTSHORTBUFLEN];
 
-      bgp_rd2str(rd_str, &ri->extra->rd);
-      kv = json_pack("{ss}", "rd", rd_str);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      if (safi == SAFI_MPLS_VPN) {
+        u_char rd_str[SHORTSHORTBUFLEN];
+
+        bgp_rd2str(rd_str, &ri->extra->rd);
+	json_object_set_new_nocheck(obj, "rd", json_string(rd_str));
+      }
+
+      bgp_label2str(label_str, ri->extra->label);
+      json_object_set_new_nocheck(obj, "label", json_string(label_str));
     }
 
     if ((bms->msglog_file && etype == BGP_LOGDUMP_ET_LOG) ||
@@ -190,6 +171,7 @@ int bgp_peer_log_msg(struct bgp_node *route, struct bgp_info *ri, safi_t safi, c
 #ifdef WITH_RABBITMQ
     if ((bms->msglog_amqp_routing_key && etype == BGP_LOGDUMP_ET_LOG) ||
 	(bms->dump_amqp_routing_key && etype == BGP_LOGDUMP_ET_DUMP)) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       amqp_ret = write_and_free_json_amqp(peer->log->amqp_host, obj);
       p_amqp_unset_routing_key(peer->log->amqp_host);
     }
@@ -198,6 +180,7 @@ int bgp_peer_log_msg(struct bgp_node *route, struct bgp_info *ri, safi_t safi, c
 #ifdef WITH_KAFKA
     if ((bms->msglog_kafka_topic && etype == BGP_LOGDUMP_ET_LOG) ||
         (bms->dump_kafka_topic && etype == BGP_LOGDUMP_ET_DUMP)) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       kafka_ret = write_and_free_json_kafka(peer->log->kafka_host, obj);
       p_kafka_unset_topic(peer->log->kafka_host);
     }
@@ -213,6 +196,7 @@ int bgp_peer_log_init(struct bgp_peer *peer, int output, int type)
   struct bgp_misc_structs *bms = bgp_select_misc_db(type);
   int peer_idx, have_it, ret = 0, amqp_ret = 0, kafka_ret = 0;
   char log_filename[SRVBUFLEN], event_type[] = "log_init";
+  pid_t writer_pid = getpid();
 
   if (!bms || !peer) return ERR;
 
@@ -270,7 +254,7 @@ int bgp_peer_log_init(struct bgp_peer *peer, int output, int type)
 
 #ifdef WITH_KAFKA
     if (bms->msglog_kafka_topic)
-      p_kafka_set_topic(peer->log->amqp_host, peer->log->filename);
+      p_kafka_set_topic(peer->log->kafka_host, peer->log->filename);
 
     if (bms->msglog_kafka_topic_rr && !p_kafka_get_topic_rr(peer->log->kafka_host)) {
       p_kafka_init_topic_rr(peer->log->kafka_host);
@@ -281,31 +265,29 @@ int bgp_peer_log_init(struct bgp_peer *peer, int output, int type)
     if (output == PRINT_OUTPUT_JSON) {
 #ifdef WITH_JANSSON
       char ip_address[INET6_ADDRSTRLEN];
-      json_t *obj = json_object(), *kv;
+      json_t *obj = json_object();
 
-      kv = json_pack("{sI}", "seq", (json_int_t)bms->log_seq);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "seq", json_integer((json_int_t)bms->log_seq));
       bgp_peer_log_seq_increment(&bms->log_seq);
 
-      kv = json_pack("{ss}", "timestamp", bms->log_tstamp_str);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "timestamp", json_string(bms->log_tstamp_str));
+
+      if (bms->bgp_peer_logdump_initclose_extras)
+	bms->bgp_peer_logdump_initclose_extras(peer, output, obj);
 
       addr_to_str(ip_address, &peer->addr);
-      kv = json_pack("{ss}", bms->peer_str, ip_address);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, bms->peer_str, json_string(ip_address));
 
-      kv = json_pack("{ss}", "event_type", event_type);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "event_type", json_string(event_type));
+
+      if (bms->bgp_peer_log_msg_extras) bms->bgp_peer_log_msg_extras(peer, output, obj);
 
       if (bms->msglog_file)
 	write_and_free_json(peer->log->fd, obj);
 
 #ifdef WITH_RABBITMQ
       if (bms->msglog_amqp_routing_key) {
+	add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
 	amqp_ret = write_and_free_json_amqp(peer->log->amqp_host, obj); 
 	p_amqp_unset_routing_key(peer->log->amqp_host);
       }
@@ -313,6 +295,7 @@ int bgp_peer_log_init(struct bgp_peer *peer, int output, int type)
 
 #ifdef WITH_KAFKA
       if (bms->msglog_kafka_topic) {
+	add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
         kafka_ret = write_and_free_json_kafka(peer->log->kafka_host, obj);
         p_kafka_unset_topic(peer->log->kafka_host);
       }
@@ -331,6 +314,7 @@ int bgp_peer_log_close(struct bgp_peer *peer, int output, int type)
   struct bgp_peer_log *log_ptr;
   void *amqp_log_ptr, *kafka_log_ptr;
   int ret = 0, amqp_ret = 0, kafka_ret = 0;
+  pid_t writer_pid = getpid();
 
   if (!bms || !peer || !peer->log) return ERR;
 
@@ -355,31 +339,27 @@ int bgp_peer_log_close(struct bgp_peer *peer, int output, int type)
   if (output == PRINT_OUTPUT_JSON) {
 #ifdef WITH_JANSSON
     char ip_address[INET6_ADDRSTRLEN];
-    json_t *obj = json_object(), *kv;
+    json_t *obj = json_object();
 
-    kv = json_pack("{sI}", "seq", (json_int_t)bms->log_seq);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "seq", json_integer((json_int_t)bms->log_seq));
     bgp_peer_log_seq_increment(&bms->log_seq);
 
-    kv = json_pack("{ss}", "timestamp", bms->log_tstamp_str);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "timestamp", json_string(bms->log_tstamp_str));
+
+    if (bms->bgp_peer_logdump_initclose_extras)
+      bms->bgp_peer_logdump_initclose_extras(peer, output, obj);
 
     addr_to_str(ip_address, &peer->addr);
-    kv = json_pack("{ss}", bms->peer_str, ip_address);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, bms->peer_str, json_string(ip_address));
 
-    kv = json_pack("{ss}", "event_type", event_type);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "event_type", json_string(event_type));
 
     if (bms->msglog_file)
       write_and_free_json(log_ptr->fd, obj);
 
 #ifdef WITH_RABBITMQ
     if (bms->msglog_amqp_routing_key) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       amqp_ret = write_and_free_json_amqp(amqp_log_ptr, obj);
       p_amqp_unset_routing_key(amqp_log_ptr);
     }
@@ -387,6 +367,7 @@ int bgp_peer_log_close(struct bgp_peer *peer, int output, int type)
 
 #ifdef WITH_KAFKA
     if (bms->msglog_kafka_topic) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       kafka_ret = write_and_free_json_kafka(kafka_log_ptr, obj);
       p_kafka_unset_topic(kafka_log_ptr);
     }
@@ -459,6 +440,7 @@ int bgp_peer_dump_init(struct bgp_peer *peer, int output, int type)
   struct bgp_misc_structs *bms = bgp_select_misc_db(type);
   char event_type[] = "dump_init";
   int ret = 0, amqp_ret = 0, kafka_ret = 0;
+  pid_t writer_pid = getpid();
 
   if (!bms || !peer || !peer->log) return ERR;
 
@@ -485,30 +467,26 @@ int bgp_peer_dump_init(struct bgp_peer *peer, int output, int type)
   if (output == PRINT_OUTPUT_JSON) {
 #ifdef WITH_JANSSON
     char ip_address[INET6_ADDRSTRLEN];
-    json_t *obj = json_object(), *kv;
+    json_t *obj = json_object();
 
-    kv = json_pack("{ss}", "timestamp", bms->dump.tstamp_str);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "timestamp", json_string(bms->dump.tstamp_str));
+
+    if (bms->bgp_peer_logdump_initclose_extras)
+      bms->bgp_peer_logdump_initclose_extras(peer, output, obj);
 
     addr_to_str(ip_address, &peer->addr);
-    kv = json_pack("{ss}", bms->peer_str, ip_address);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, bms->peer_str, json_string(ip_address));
 
-    kv = json_pack("{ss}", "event_type", event_type);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "event_type", json_string(event_type));
 
-    kv = json_pack("{sI}", "dump_period", (json_int_t)bms->dump.period);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "dump_period", json_integer((json_int_t)bms->dump.period));
 
     if (bms->dump_file)
       write_and_free_json(peer->log->fd, obj);
 
 #ifdef WITH_RABBITMQ
     if (bms->dump_amqp_routing_key) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       amqp_ret = write_and_free_json_amqp(peer->log->amqp_host, obj);
       p_amqp_unset_routing_key(peer->log->amqp_host);
     }
@@ -516,6 +494,7 @@ int bgp_peer_dump_init(struct bgp_peer *peer, int output, int type)
 
 #ifdef WITH_KAFKA
     if (bms->dump_kafka_topic) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       kafka_ret = write_and_free_json_kafka(peer->log->kafka_host, obj);
       p_kafka_unset_topic(peer->log->kafka_host);
     }
@@ -531,6 +510,7 @@ int bgp_peer_dump_close(struct bgp_peer *peer, struct bgp_dump_stats *bds, int o
   struct bgp_misc_structs *bms = bgp_select_misc_db(type);
   char event_type[] = "dump_close";
   int ret = 0, amqp_ret = 0, kafka_ret = 0;
+  pid_t writer_pid = getpid();
 
   if (!bms || !peer || !peer->log) return ERR;
 
@@ -549,27 +529,20 @@ int bgp_peer_dump_close(struct bgp_peer *peer, struct bgp_dump_stats *bds, int o
     char ip_address[INET6_ADDRSTRLEN];
     json_t *obj = json_object(), *kv;
 
-    kv = json_pack("{ss}", "timestamp", bms->dump.tstamp_str);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "timestamp", json_string(bms->dump.tstamp_str));
+
+    if (bms->bgp_peer_logdump_initclose_extras)
+      bms->bgp_peer_logdump_initclose_extras(peer, output, obj);
 
     addr_to_str(ip_address, &peer->addr);
-    kv = json_pack("{ss}", bms->peer_str, ip_address);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, bms->peer_str, json_string(ip_address));
 
-    kv = json_pack("{ss}", "event_type", event_type);
-    json_object_update_missing(obj, kv);
-    json_decref(kv);
+    json_object_set_new_nocheck(obj, "event_type", json_string(event_type));
 
     if (bds) {
-      kv = json_pack("{sI}", "entries", (json_int_t)bds->entries);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "entries", json_integer((json_int_t)bds->entries));
 
-      kv = json_pack("{sI}", "tables", (json_int_t)bds->tables);
-      json_object_update_missing(obj, kv);
-      json_decref(kv);
+      json_object_set_new_nocheck(obj, "tables", json_integer((json_int_t)bds->tables));
     }
 
     if (bms->dump_file)
@@ -577,6 +550,7 @@ int bgp_peer_dump_close(struct bgp_peer *peer, struct bgp_dump_stats *bds, int o
 
 #ifdef WITH_RABBITMQ
     if (bms->dump_amqp_routing_key) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       amqp_ret = write_and_free_json_amqp(peer->log->amqp_host, obj);
       p_amqp_unset_routing_key(peer->log->amqp_host);
     }
@@ -584,6 +558,7 @@ int bgp_peer_dump_close(struct bgp_peer *peer, struct bgp_dump_stats *bds, int o
 
 #ifdef WITH_KAFKA
     if (bms->dump_kafka_topic) {
+      add_writer_name_and_pid_json(obj, config.proc_name, writer_pid);
       kafka_ret = write_and_free_json_kafka(peer->log->kafka_host, obj);
       p_kafka_unset_topic(peer->log->kafka_host);
     }
@@ -718,14 +693,14 @@ void bgp_handle_dump_event()
 	    node = bgp_table_top(peer, table);
 
 	    while (node) {
-	      u_int32_t modulo = bgp_route_info_modulo(peer, NULL);
+	      u_int32_t modulo = bgp_route_info_modulo(peer, NULL, bms->table_per_peer_buckets);
 	      u_int32_t peer_buckets;
 	      struct bgp_info *ri;
 
 	      for (peer_buckets = 0; peer_buckets < config.bgp_table_per_peer_buckets; peer_buckets++) {
 	        for (ri = node->info[modulo+peer_buckets]; ri; ri = ri->next) {
 		  if (ri->peer == peer) {
-	            bgp_peer_log_msg(node, ri, safi, event_type, config.bgp_table_dump_output, BGP_LOG_TYPE_MISC);
+	            bgp_peer_log_msg(node, ri, afi, safi, event_type, config.bgp_table_dump_output, BGP_LOG_TYPE_MISC);
 	            dump_elems++;
 		  }
 		}
@@ -840,7 +815,7 @@ int bgp_daemon_msglog_init_kafka_host()
 {
   int ret;
 
-  p_kafka_init_host(&bgp_daemon_msglog_kafka_host);
+  p_kafka_init_host(&bgp_daemon_msglog_kafka_host, config.nfacctd_bgp_msglog_kafka_config_file);
   ret = p_kafka_connect_to_produce(&bgp_daemon_msglog_kafka_host);
 
   if (!config.nfacctd_bgp_msglog_kafka_broker_host) config.nfacctd_bgp_msglog_kafka_broker_host = default_kafka_broker_host;
@@ -868,7 +843,7 @@ int bgp_table_dump_init_kafka_host()
 {
   int ret;
 
-  p_kafka_init_host(&bgp_table_dump_kafka_host);
+  p_kafka_init_host(&bgp_table_dump_kafka_host, config.bgp_table_dump_kafka_config_file);
   ret = p_kafka_connect_to_produce(&bgp_table_dump_kafka_host);
 
   if (!config.bgp_table_dump_kafka_broker_host) config.bgp_table_dump_kafka_broker_host = default_kafka_broker_host;
