@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -42,6 +42,9 @@ EXT int cfg_key_aggregate_primitives(char *, char *, char *);
 EXT int cfg_key_snaplen(char *, char *, char *);
 EXT int cfg_key_aggregate_filter(char *, char *, char *);
 EXT int cfg_key_pcap_filter(char *, char *, char *);
+EXT int cfg_key_pcap_protocol(char *, char *, char *);
+EXT int cfg_key_pcap_savefile(char *, char *, char *);
+EXT int cfg_key_pcap_savefile_wait(char *, char *, char *);
 EXT int cfg_key_use_ip_next_hop(char *, char *, char *);
 EXT int cfg_key_thread_stack(char *, char *, char *);
 EXT int cfg_key_interface(char *, char *, char *);
@@ -49,7 +52,6 @@ EXT int cfg_key_interface_wait(char *, char *, char *);
 EXT int cfg_key_files_umask(char *, char *, char *);
 EXT int cfg_key_files_uid(char *, char *, char *);
 EXT int cfg_key_files_gid(char *, char *, char *);
-EXT int cfg_key_savefile_wait(char *, char *, char *);
 EXT int cfg_key_promisc(char *, char *, char *);
 EXT int cfg_key_num_protos(char *, char *, char *);
 EXT int cfg_key_num_hosts(char *, char *, char *);
@@ -74,7 +76,6 @@ EXT int cfg_key_sql_history(char *, char *, char *);
 EXT int cfg_key_sql_history_offset(char *, char *, char *);
 EXT int cfg_key_sql_history_roundoff(char *, char *, char *);
 EXT int cfg_key_sql_recovery_backup_host(char *, char *, char *);
-EXT int cfg_key_sql_max_writers(char *, char *, char *);
 EXT int cfg_key_sql_trigger_exec(char *, char *, char *);
 EXT int cfg_key_sql_trigger_time(char *, char *, char *);
 EXT int cfg_key_sql_cache_entries(char *, char *, char *);
@@ -82,7 +83,6 @@ EXT int cfg_key_sql_dont_try_update(char *, char *, char *);
 EXT int cfg_key_sql_preprocess(char *, char *, char *);
 EXT int cfg_key_sql_preprocess_type(char *, char *, char *);
 EXT int cfg_key_sql_multi_values(char *, char *, char *);
-EXT int cfg_key_sql_aggressive_classification(char *, char *, char *);
 EXT int cfg_key_sql_locking_style(char *, char *, char *);
 EXT int cfg_key_sql_use_copy(char *, char *, char *);
 EXT int cfg_key_sql_delimiter(char *, char *, char *);
@@ -98,31 +98,25 @@ EXT int cfg_key_amqp_frame_max(char *, char *, char *);
 EXT int cfg_key_amqp_heartbeat_interval(char *, char *, char *);
 EXT int cfg_key_amqp_vhost(char *, char *, char *);
 EXT int cfg_key_amqp_routing_key_rr(char *, char *, char *);
+EXT int cfg_key_amqp_avro_schema_routing_key(char *, char *, char *);
+EXT int cfg_key_amqp_avro_schema_refresh_time(char *, char *, char *);
 EXT int cfg_key_kafka_broker_port(char *, char *, char *);
 EXT int cfg_key_kafka_partition(char *, char *, char *);
 EXT int cfg_key_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_kafka_avro_schema_topic(char *, char *, char *);
+EXT int cfg_key_kafka_avro_schema_refresh_time(char *, char *, char *);
+EXT int cfg_key_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_plugin_pipe_size(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_backlog(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_check_core_pid(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_user(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_passwd(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_exchange(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_host(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_vhost(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_routing_key(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_amqp_retry(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_kafka(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_kafka_broker_host(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_kafka_broker_port(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_kafka_topic(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_kafka_partition(char *, char *, char *);
-EXT int cfg_key_plugin_pipe_kafka_retry(char *, char *, char *);
 EXT int cfg_key_plugin_buffer_size(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_check_core_pid(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_zmq(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_zmq_retry(char *, char *, char *);
+EXT int cfg_key_plugin_pipe_zmq_profile(char *, char *, char *);
 EXT int cfg_key_networks_mask(char *, char *, char *);
 EXT int cfg_key_networks_file(char *, char *, char *);
 EXT int cfg_key_networks_file_filter(char *, char *, char *);
 EXT int cfg_key_networks_file_no_lpm(char *, char *, char *);
+EXT int cfg_key_networks_no_mask_if_zero(char *, char *, char *);
 EXT int cfg_key_networks_cache_entries(char *, char *, char *);
 EXT int cfg_key_ports_file(char *, char *, char *);
 EXT int cfg_key_print_cache_entries(char *, char *, char *);
@@ -141,9 +135,11 @@ EXT int cfg_key_nfacctd_time_new(char *, char *, char *);
 EXT int cfg_key_nfacctd_as_new(char *, char *, char *);
 EXT int cfg_key_nfacctd_net(char *, char *, char *);
 EXT int cfg_key_nfacctd_disable_checks(char *, char *, char *);
+EXT int cfg_key_nfacctd_disable_opt_scope_check(char *, char *, char *);
 EXT int cfg_key_nfacctd_mcast_groups(char *, char *, char *);
 EXT int cfg_key_nfacctd_pipe_size(char *, char *, char *);
 EXT int cfg_key_nfacctd_pro_rating(char *, char *, char *);
+EXT int cfg_key_nfacctd_templates_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_account_options(char *, char *, char *);
 EXT int cfg_key_nfacctd_stitching(char *, char *, char *);
 EXT int cfg_key_pmacctd_force_frag_handling(char *, char *, char *);
@@ -175,6 +171,7 @@ EXT int cfg_key_sfacctd_counter_kafka_topic(char *, char *, char *);
 EXT int cfg_key_sfacctd_counter_kafka_partition(char *, char *, char *);
 EXT int cfg_key_sfacctd_counter_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_sfacctd_counter_kafka_retry(char *, char *, char *);
+EXT int cfg_key_sfacctd_counter_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_telemetry_daemon(char *, char *, char *);
 EXT int cfg_key_telemetry_port_tcp(char *, char *, char *);
 EXT int cfg_key_telemetry_port_udp(char *, char *, char *);
@@ -206,6 +203,7 @@ EXT int cfg_key_telemetry_msglog_kafka_topic_rr(char *, char *, char *);
 EXT int cfg_key_telemetry_msglog_kafka_partition(char *, char *, char *);
 EXT int cfg_key_telemetry_msglog_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_telemetry_msglog_kafka_retry(char *, char *, char *);
+EXT int cfg_key_telemetry_msglog_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_telemetry_dump_output(char *, char *, char *);
 EXT int cfg_key_telemetry_dump_file(char *, char *, char *);
 EXT int cfg_key_telemetry_dump_latest_file(char *, char *, char *);
@@ -227,7 +225,7 @@ EXT int cfg_key_telemetry_dump_kafka_topic(char *, char *, char *);
 EXT int cfg_key_telemetry_dump_kafka_topic_rr(char *, char *, char *);
 EXT int cfg_key_telemetry_dump_kafka_partition(char *, char *, char *);
 EXT int cfg_key_telemetry_dump_kafka_partition_key(char *, char *, char *);
-EXT int cfg_key_pcap_savefile(char *, char *, char *);
+EXT int cfg_key_telemetry_dump_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_maps_refresh(char *, char *, char *);
 EXT int cfg_key_maps_index(char *, char *, char *);
 EXT int cfg_key_maps_entries(char *, char *, char *);
@@ -243,6 +241,15 @@ EXT int cfg_key_sampling_map(char *, char *, char *);
 EXT int cfg_key_classifiers(char *, char *, char *);
 EXT int cfg_key_classifier_tentatives(char *, char *, char *);
 EXT int cfg_key_classifier_table_num(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_num_roots(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_max_flows(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_proto_guess(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_idle_scan_period(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_idle_max_time(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_idle_scan_budget(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_giveup_proto_tcp(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_giveup_proto_udp(char *, char *, char *);
+EXT int cfg_key_classifier_ndpi_giveup_proto_other(char *, char *, char *);
 EXT int cfg_key_nfprobe_timeouts(char *, char *, char *);
 EXT int cfg_key_nfprobe_hoplimit(char *, char *, char *);
 EXT int cfg_key_nfprobe_maxflows(char *, char *, char *);
@@ -263,6 +270,7 @@ EXT int cfg_key_tee_transparent(char *, char *, char *);
 EXT int cfg_key_tee_max_receivers(char *, char *, char *);
 EXT int cfg_key_tee_max_receiver_pools(char *, char *, char *);
 EXT int cfg_key_tee_pipe_size(char *, char *, char *);
+EXT int cfg_key_tee_dissect_send_full_pkt(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_output(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_file(char *, char *, char *);
@@ -285,19 +293,23 @@ EXT int cfg_key_nfacctd_bgp_msglog_kafka_topic_rr(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_kafka_partition(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_msglog_kafka_retry(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_msglog_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_max_peers(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_ip(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_id(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_as(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_port(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_ip_precedence(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_allow_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_aspath_radius(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_stdcomm_pattern(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_extcomm_pattern(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_lrgcomm_pattern(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_stdcomm_pattern_to_asn(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_peer_src_as_type(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_std_comm_type(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_ext_comm_type(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_src_lrg_comm_type(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_as_path_type(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_local_pref_type(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_src_med_type(char *, char *, char *);
@@ -336,9 +348,13 @@ EXT int cfg_key_nfacctd_bgp_table_dump_kafka_topic(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_kafka_topic_rr(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_kafka_partition(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_table_dump_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_table_dump_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_batch(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_batch_interval(char *, char *, char *);
 EXT int cfg_key_nfacctd_bgp_pipe_size(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_offline_input(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_offline_file_spool(char *, char *, char *);
+EXT int cfg_key_nfacctd_bgp_offline_file_refresh_time(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_ip(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_port(char *, char *, char *);
@@ -369,6 +385,7 @@ EXT int cfg_key_nfacctd_bmp_msglog_kafka_topic_rr(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_kafka_partition(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_kafka_partition_key(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_msglog_kafka_retry(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_msglog_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_peer_buckets(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_per_peer_buckets(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_table_attr_hash_buckets(char *, char *, char *);
@@ -394,6 +411,7 @@ EXT int cfg_key_nfacctd_bmp_dump_kafka_topic(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_kafka_topic_rr(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_kafka_partition(char *, char *, char *);
 EXT int cfg_key_nfacctd_bmp_dump_kafka_partition_key(char *, char *, char *);
+EXT int cfg_key_nfacctd_bmp_dump_kafka_config_file(char *, char *, char *);
 EXT int cfg_key_nfacctd_flow_to_rd_map(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis(char *, char *, char *);
 EXT int cfg_key_nfacctd_isis_ip(char *, char *, char *);
@@ -411,9 +429,8 @@ EXT int cfg_key_uacctd_nl_size(char *, char *, char *);
 EXT int cfg_key_uacctd_threshold(char *, char *, char *);
 EXT int cfg_key_tunnel_0(char *, char *, char *);
 EXT int cfg_key_pkt_len_distrib_bins(char *, char *, char *);
-EXT int cfg_key_tmp_net_own_field(char *, char *, char *);
+EXT int cfg_key_dump_max_writers(char *, char *, char *);
 EXT int cfg_key_tmp_asa_bi_flow(char *, char *, char *);
-EXT int cfg_key_tmp_comms_same_field(char *, char *, char *);
 
 EXT void parse_time(char *, char *, int *, int *);
 EXT void cfg_set_aggregate(char *, u_int64_t [], u_int64_t, char *);
