@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -42,6 +42,7 @@ struct acc {
   struct cache_legacy_bgp_primitives *clbgp;
   struct pkt_nat_primitives *pnat;
   struct pkt_mpls_primitives *pmpls;
+  struct pkt_tunnel_primitives *ptun;
   char *pcust;
   struct pkt_vlen_hdr_primitives *pvlen;
   struct acc *next;
@@ -81,6 +82,7 @@ struct query_entry {
   struct pkt_legacy_bgp_primitives plbgp;	/* extended BGP data */
   struct pkt_nat_primitives pnat;		/* extended NAT + timestamp data */
   struct pkt_mpls_primitives pmpls;		/* extended MPLS data */
+  struct pkt_tunnel_primitives ptun;		/* extended tunnel data */
   char *pcust;					/* custom-defined data */
   struct pkt_vlen_hdr_primitives *pvlen;	/* variable-length data */
 };
@@ -150,8 +152,8 @@ EXT void reset_counters(struct acc *);
 EXT int build_query_server(char *);
 EXT void process_query_data(int, unsigned char *, int, struct extra_primitives *, int, int);
 EXT void mask_elem(struct pkt_primitives *, struct pkt_bgp_primitives *, struct pkt_legacy_bgp_primitives *,
-			struct pkt_nat_primitives *, struct pkt_mpls_primitives *, struct acc *, u_int64_t,
-			u_int64_t, struct extra_primitives *);
+			struct pkt_nat_primitives *, struct pkt_mpls_primitives *, struct pkt_tunnel_primitives *,
+			struct acc *, u_int64_t, u_int64_t, struct extra_primitives *);
 EXT void enQueue_elem(int, struct reply_buffer *, void *, int, int);
 EXT void Accumulate_Counters(struct pkt_data *, struct acc *);
 EXT int test_zero_elem(struct acc *);
