@@ -71,12 +71,12 @@ isis_nexthop_create (struct in_addr *ip, unsigned int ifindex)
   nexthop = calloc(1, sizeof (struct isis_nexthop));
   if (!nexthop)
     {
-      Log(LOG_ERR, "ERROR ( default/core/ISIS ): ISIS-Rte: isis_nexthop_create: out of memory!\n");
+      Log(LOG_ERR, "ERROR ( %s/core/ISIS ): ISIS-Rte: isis_nexthop_create: out of memory!\n", config.name);
     }
 
   nexthop->ifindex = ifindex;
   memcpy (&nexthop->ip, ip, sizeof (struct in_addr));
-  listnode_add (isis->nexthops, nexthop);
+  isis_listnode_add (isis->nexthops, nexthop);
   nexthop->lock++;
 
   return nexthop;
@@ -88,7 +88,7 @@ isis_nexthop_delete (struct isis_nexthop *nexthop)
   nexthop->lock--;
   if (nexthop->lock == 0)
     {
-      listnode_delete (isis->nexthops, nexthop);
+      isis_listnode_delete (isis->nexthops, nexthop);
       free(nexthop);
     }
 
@@ -121,7 +121,7 @@ isis_nexthop6_new (struct in6_addr *ip6, unsigned int ifindex)
   nexthop6 = calloc(1, sizeof (struct isis_nexthop6));
   if (!nexthop6)
     {
-      Log(LOG_ERR, "ERROR ( default/core/ISIS ): ISIS-Rte: isis_nexthop_create6: out of memory!\n");
+      Log(LOG_ERR, "ERROR ( %s/core/ISIS ): ISIS-Rte: isis_nexthop_create6: out of memory!\n", config.name);
     }
 
   nexthop6->ifindex = ifindex;
@@ -160,7 +160,7 @@ isis_nexthop6_delete (struct isis_nexthop6 *nexthop6)
   nexthop6->lock--;
   if (nexthop6->lock == 0)
     {
-      listnode_delete (isis->nexthops6, nexthop6);
+      isis_listnode_delete (isis->nexthops6, nexthop6);
       free(nexthop6);
     }
 
@@ -201,7 +201,7 @@ adjinfo2nexthop (struct list *nexthops, struct isis_adjacency *adj)
       if (!nexthoplookup (nexthops, ipv4_addr, adj->circuit->interface->ifindex))
 	{
 	  nh = isis_nexthop_create (ipv4_addr, adj->circuit->interface->ifindex);
-	  listnode_add (nexthops, nh);
+	  isis_listnode_add (nexthops, nh);
 	}
     }
 }
@@ -224,7 +224,7 @@ adjinfo2nexthop6 (struct list *nexthops6, struct isis_adjacency *adj)
 	{
 	  nh6 = isis_nexthop6_create (ipv6_addr,
 				      adj->circuit->interface->ifindex);
-	  listnode_add (nexthops6, nh6);
+	  isis_listnode_add (nexthops6, nh6);
 	}
     }
 }
@@ -241,20 +241,20 @@ isis_route_info_new (uint32_t cost, uint32_t depth, u_char family,
   rinfo = calloc(1, sizeof (struct isis_route_info));
   if (!rinfo)
     {
-      Log(LOG_ERR, "ERROR ( default/core/ISIS ): ISIS-Rte: isis_route_info_new: out of memory!\n");
+      Log(LOG_ERR, "ERROR ( %s/core/ISIS ): ISIS-Rte: isis_route_info_new: out of memory!\n", config.name);
       return NULL;
     }
 
   if (family == AF_INET)
     {
-      rinfo->nexthops = list_new ();
+      rinfo->nexthops = isis_list_new ();
       for (ALL_LIST_ELEMENTS_RO (adjacencies, node, adj))
         adjinfo2nexthop (rinfo->nexthops, adj);
     }
 #ifdef ENABLE_IPV6
   if (family == AF_INET6)
     {
-      rinfo->nexthops6 = list_new ();
+      rinfo->nexthops6 = isis_list_new ();
       for (ALL_LIST_ELEMENTS_RO (adjacencies, node, adj))
         adjinfo2nexthop6 (rinfo->nexthops6, adj);
     }
@@ -273,14 +273,14 @@ isis_route_info_delete (struct isis_route_info *route_info)
   if (route_info->nexthops)
     {
       route_info->nexthops->del = (void (*)(void *)) isis_nexthop_delete;
-      list_delete (route_info->nexthops);
+      isis_list_delete (route_info->nexthops);
     }
 
 #ifdef ENABLE_IPV6
   if (route_info->nexthops6)
     {
       route_info->nexthops6->del = (void (*)(void *)) isis_nexthop6_delete;
-      list_delete (route_info->nexthops6);
+      isis_list_delete (route_info->nexthops6);
     }
 #endif /* ENABLE_IPV6 */
 
@@ -351,7 +351,7 @@ isis_nexthops_merge (struct list *new, struct list *old)
     {
       if (nexthoplookup (old, &nexthop->ip, nexthop->ifindex))
 	continue;
-      listnode_add (old, nexthop);
+      isis_listnode_add (old, nexthop);
       nexthop->lock++;
     }
 }
@@ -367,7 +367,7 @@ isis_nexthops6_merge (struct list *new, struct list *old)
     {
       if (nexthop6lookup (old, &nexthop6->ip6, nexthop6->ifindex))
 	continue;
-      listnode_add (old, nexthop6);
+      isis_listnode_add (old, nexthop6);
       nexthop6->lock++;
     }
 }
@@ -417,8 +417,8 @@ isis_route_create (struct isis_prefix *prefix, u_int32_t cost, u_int32_t depth,
   rinfo_new = isis_route_info_new (cost, depth, family, adjacencies);
   if (!rinfo_new)
     {
-      Log(LOG_ERR, "ERROR ( default/core/ISIS ): ISIS-Rte (%s): isis_route_create: out of memory!\n",
-		area->area_tag);
+      Log(LOG_ERR, "ERROR ( %s/core/ISIS ): ISIS-Rte (%s): isis_route_create: out of memory!\n",
+		config.name, area->area_tag);
       return NULL;
     }
 
@@ -434,20 +434,22 @@ isis_route_create (struct isis_prefix *prefix, u_int32_t cost, u_int32_t depth,
   if (!rinfo_old)
     {
       if (config.nfacctd_isis_msglog)
-        Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route created: %s\n", area->area_tag, area->is_type, buff);
+        Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route created: %s\n",
+		config.name, area->area_tag, area->is_type, buff);
       SET_FLAG (rinfo_new->flag, ISIS_ROUTE_FLAG_ACTIVE);
       route_node->info = rinfo_new;
       return rinfo_new;
     }
 
   if (config.nfacctd_isis_msglog)
-    Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route already exists: %s\n",
-	area->area_tag, area->is_type, buff);
+    Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route already exists: %s\n",
+		config.name, area->area_tag, area->is_type, buff);
 
   if (isis_route_info_same (rinfo_new, rinfo_old, family))
     {
       if (config.nfacctd_isis_msglog)
-        Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route unchanged: %s\n", area->area_tag, area->is_type, buff);
+        Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route unchanged: %s\n",
+		config.name, area->area_tag, area->is_type, buff);
       isis_route_info_delete (rinfo_new);
       route_info = rinfo_old;
     }
@@ -455,8 +457,8 @@ isis_route_create (struct isis_prefix *prefix, u_int32_t cost, u_int32_t depth,
     {
       /* merge the nexthop lists */
       if (config.nfacctd_isis_msglog)
-        Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route changed (same attribs): %s\n",
-		   area->area_tag, area->is_type, buff);
+        Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route changed (same attribs): %s\n",
+		   config.name, area->area_tag, area->is_type, buff);
       isis_route_info_merge (rinfo_new, rinfo_old, family);
       isis_route_info_delete (rinfo_new);
       route_info = rinfo_old;
@@ -467,16 +469,16 @@ isis_route_create (struct isis_prefix *prefix, u_int32_t cost, u_int32_t depth,
       if (isis_route_info_prefer_new (rinfo_new, rinfo_old))
 	{
 	  if (config.nfacctd_isis_msglog)
-	    Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route changed: %s\n",
-		area->area_tag, area->is_type, buff);
+	    Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route changed: %s\n",
+			config.name, area->area_tag, area->is_type, buff);
 	  isis_route_info_delete (rinfo_old);
 	  route_info = rinfo_new;
 	}
       else
 	{
 	  if (config.nfacctd_isis_msglog)
-	    Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route rejected: %s\n",
-		area->area_tag, area->is_type, buff);
+	    Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u) route rejected: %s\n",
+			config.name, area->area_tag, area->is_type, buff);
 	  isis_route_info_delete (rinfo_new);
 	  route_info = rinfo_old;
 	}
@@ -504,7 +506,7 @@ isis_route_delete (struct isis_prefix *prefix, struct route_table *table)
 
   if (rinfo == NULL)
     {
-      Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte: tried to delete non-existant route: %s\n", buff);
+      Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte: tried to delete non-existant route: %s\n", config.name, buff);
 
       return;
     }
@@ -532,8 +534,8 @@ void isis_route_validate_table (struct isis_area *area, struct route_table *tabl
 	{
 	  isis_prefix2str (&rnode->p, (char *) buff, BUFSIZ);
 	  if (config.nfacctd_isis_msglog)
-	    Log(LOG_DEBUG, "DEBUG ( default/core/ISIS ): ISIS-Rte (tag: %s, level: %u): route validate: %s %s\n",
-		      area->area_tag, area->is_type,
+	    Log(LOG_DEBUG, "DEBUG ( %s/core/ISIS ): ISIS-Rte (tag: %s, level: %u): route validate: %s %s\n",
+		      config.name, area->area_tag, area->is_type,
 		      (CHECK_FLAG (rinfo->flag, ISIS_ROUTE_FLAG_ACTIVE) ?
 		      "active" : "inactive"), buff);
 	}
