@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -34,22 +34,8 @@
 #define EXT
 #endif
 EXT void kafka_plugin(int, struct configuration *, void *);
-EXT void kafka_cache_purge(struct chained_cache *[], int);
-
-/* global vars */
-EXT void (*insert_func)(struct primitives_ptrs *, struct insert_data *); /* pointer to INSERT function */
-EXT void (*purge_func)(struct chained_cache *[], int); /* pointer to purge function */
-EXT struct scratch_area sa;
-EXT struct chained_cache *cache;
-EXT struct chained_cache **queries_queue;
-EXT struct timeval flushtime;
-EXT int qq_ptr, pp_size, pb_size, pn_size, pm_size, dbc_size, quit;
-EXT time_t refresh_deadline;
-
-EXT struct timeval sbasetime;
-
+EXT void kafka_cache_purge(struct chained_cache *[], int, int);
 #ifdef WITH_AVRO
-EXT char *avro_buf;
-EXT avro_schema_t avro_acct_schema;
+EXT void kafka_avro_schema_purge(char *);
 #endif
 #undef EXT
