@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -23,6 +23,7 @@
 
 /* includes */
 #include "pmacct.h"
+#include "addr.h"
 #include "pmacct-data.h"
 #include "plugin_common.h"
 #include "ip_flow.h"
@@ -67,15 +68,16 @@ void P_init_default_values()
 
   if (!config.sql_refresh_time) config.sql_refresh_time = DEFAULT_PLUGIN_COMMON_REFRESH_TIME;
   if (!config.print_cache_entries) config.print_cache_entries = PRINT_CACHE_ENTRIES;
-  if (!config.sql_max_writers) config.sql_max_writers = DEFAULT_PLUGIN_COMMON_WRITERS_NO;
+  if (!config.dump_max_writers) config.dump_max_writers = DEFAULT_PLUGIN_COMMON_WRITERS_NO;
 
-  dump_writers.list = malloc(config.sql_max_writers * sizeof(pid_t));
+  dump_writers.list = malloc(config.dump_max_writers * sizeof(pid_t));
   dump_writers_init();
 
   pp_size = sizeof(struct pkt_primitives);
   pb_size = sizeof(struct pkt_bgp_primitives);
   pn_size = sizeof(struct pkt_nat_primitives);
   pm_size = sizeof(struct pkt_mpls_primitives);
+  pt_size = sizeof(struct pkt_tunnel_primitives);
   pc_size = config.cpptrs.len;
   dbc_size = sizeof(struct chained_cache);
 
@@ -124,6 +126,7 @@ unsigned int P_cache_modulo(struct primitives_ptrs *prim_ptrs)
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
   struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
+  struct pkt_tunnel_primitives *ptun = prim_ptrs->ptun;
   char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   register unsigned int modulo;
@@ -132,6 +135,7 @@ unsigned int P_cache_modulo(struct primitives_ptrs *prim_ptrs)
   if (pbgp) modulo ^= cache_crc32((unsigned char *)pbgp, pb_size);
   if (pnat) modulo ^= cache_crc32((unsigned char *)pnat, pn_size);
   if (pmpls) modulo ^= cache_crc32((unsigned char *)pmpls, pm_size);
+  if (ptun) modulo ^= cache_crc32((unsigned char *)ptun, pt_size);
   if (pcust) modulo ^= cache_crc32((unsigned char *)pcust, pc_size);
   if (pvlen) modulo ^= cache_crc32((unsigned char *)pvlen, (PvhdrSz + pvlen->tot_len));
 
@@ -145,12 +149,13 @@ struct chained_cache *P_cache_search(struct primitives_ptrs *prim_ptrs)
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
   struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
+  struct pkt_tunnel_primitives *ptun = prim_ptrs->ptun;
   char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   unsigned int modulo = P_cache_modulo(prim_ptrs);
   struct chained_cache *cache_ptr = &cache[modulo];
-  int res_data = TRUE, res_bgp = TRUE, res_nat = TRUE, res_mpls = TRUE, res_time = TRUE;
-  int res_cust = TRUE, res_vlen = TRUE;
+  int res_data = TRUE, res_bgp = TRUE, res_nat = TRUE, res_mpls = TRUE, res_tun = TRUE;
+  int res_time = TRUE, res_cust = TRUE, res_vlen = TRUE;
 
   start:
   res_data = memcmp(&cache_ptr->primitives, data, sizeof(struct pkt_primitives));
@@ -175,6 +180,11 @@ struct chained_cache *P_cache_search(struct primitives_ptrs *prim_ptrs)
   }
   else res_mpls = FALSE;
 
+  if (ptun) {
+    if (cache_ptr->ptun) res_tun = memcmp(cache_ptr->ptun, ptun, sizeof(struct pkt_tunnel_primitives));
+  }
+  else res_tun = FALSE;
+
   if (pcust) {
     if (cache_ptr->pcust) res_cust = memcmp(cache_ptr->pcust, pcust, config.cpptrs.len);
   }
@@ -185,7 +195,7 @@ struct chained_cache *P_cache_search(struct primitives_ptrs *prim_ptrs)
   }
   else res_vlen = FALSE;
 
-  if (res_data || res_bgp || res_nat || res_mpls || res_time || res_cust || res_vlen) {
+  if (res_data || res_bgp || res_nat || res_mpls || res_tun || res_time || res_cust || res_vlen) {
     if (cache_ptr->valid == PRINT_CACHE_INUSE) {
       if (cache_ptr->next) {
         cache_ptr = cache_ptr->next;
@@ -204,12 +214,13 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
   struct pkt_bgp_primitives *pbgp = prim_ptrs->pbgp;
   struct pkt_nat_primitives *pnat = prim_ptrs->pnat;
   struct pkt_mpls_primitives *pmpls = prim_ptrs->pmpls;
+  struct pkt_tunnel_primitives *ptun = prim_ptrs->ptun;
   char *pcust = prim_ptrs->pcust;
   struct pkt_vlen_hdr_primitives *pvlen = prim_ptrs->pvlen;
   unsigned int modulo = P_cache_modulo(prim_ptrs);
   struct chained_cache *cache_ptr = &cache[modulo];
   struct pkt_primitives *srcdst = &data->primitives;
-  int res_data, res_bgp, res_nat, res_mpls, res_time, res_cust, res_vlen;
+  int res_data, res_bgp, res_nat, res_mpls, res_tun, res_time, res_cust, res_vlen;
 
   /* pro_rating vars */
   int time_delta = 0, time_total = 0;
@@ -270,7 +281,7 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
   }
 
   start:
-  res_data = res_bgp = res_nat = res_mpls = res_time = res_cust = res_vlen = TRUE;
+  res_data = res_bgp = res_nat = res_mpls = res_tun = res_time = res_cust = res_vlen = TRUE;
 
   res_data = memcmp(&cache_ptr->primitives, srcdst, sizeof(struct pkt_primitives)); 
 
@@ -294,6 +305,11 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
   }
   else res_mpls = FALSE;
 
+  if (ptun) {
+    if (cache_ptr->ptun) res_tun = memcmp(cache_ptr->ptun, ptun, sizeof(struct pkt_tunnel_primitives));
+  }
+  else res_tun = FALSE;
+
   if (pcust) {
     if (cache_ptr->pcust) res_cust = memcmp(cache_ptr->pcust, pcust, config.cpptrs.len); 
   }
@@ -304,7 +320,7 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
   }
   else res_vlen = FALSE;
 
-  if (res_data || res_bgp || res_nat || res_mpls || res_time || res_cust || res_vlen) {
+  if (res_data || res_bgp || res_nat || res_mpls || res_tun || res_time || res_cust || res_vlen) {
     /* aliasing of entries */
     if (cache_ptr->valid == PRINT_CACHE_INUSE) { 
       if (cache_ptr->next) {
@@ -357,6 +373,16 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
       cache_ptr->pmpls = NULL;
     }
 
+    if (ptun) {
+      if (!cache_ptr->ptun) cache_ptr->ptun = (struct pkt_tunnel_primitives *) malloc(PtunSz);
+      if (cache_ptr->ptun) memcpy(cache_ptr->ptun, ptun, sizeof(struct pkt_tunnel_primitives));
+      else goto safe_action;
+    }
+    else {
+      if (cache_ptr->ptun) free(cache_ptr->ptun);
+      cache_ptr->ptun = NULL;
+    }
+
     if (pcust) {
       if (!cache_ptr->pcust) cache_ptr->pcust = malloc(config.cpptrs.len);
       if (cache_ptr->pcust) memcpy(cache_ptr->pcust, pcust, config.cpptrs.len);
@@ -436,7 +462,9 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
       if (config.nfacctd_stitching) {
 	if (cache_ptr->stitch) {
 	  if (data->time_end.tv_sec) {
-	    memcpy(&cache_ptr->stitch->timestamp_max, &data->time_end, sizeof(struct timeval));
+	    if (data->time_end.tv_sec > cache_ptr->stitch->timestamp_max.tv_sec && 
+		data->time_end.tv_usec > cache_ptr->stitch->timestamp_max.tv_usec)
+	      memcpy(&cache_ptr->stitch->timestamp_max, &data->time_end, sizeof(struct timeval));
 	  }
 	  else {
 	    cache_ptr->stitch->timestamp_max.tv_sec = idata->now;
@@ -491,7 +519,8 @@ void P_cache_insert(struct primitives_ptrs *prim_ptrs, struct insert_data *idata
     if (dump_writers_get_flags() != CHLD_ALERT) {
       switch (ret = fork()) {
       case 0: /* Child */
-        (*purge_func)(queries_queue, qq_ptr);
+	pm_setproctitle("%s %s [%s]", config.type, "Plugin -- Writer (urgent)", config.name);
+        (*purge_func)(queries_queue, qq_ptr, TRUE);
         exit(0);
       default: /* Parent */
         if (ret == -1) Log(LOG_WARNING, "WARN ( %s/%s ): Unable to fork writer: %s\n", config.name, config.type, strerror(errno));
@@ -556,8 +585,9 @@ void P_cache_insert_pending(struct chained_cache *queue[], int index, struct cha
     }
 
     if (cache_ptr->pbgp) free(cache_ptr->pbgp);
-    if (cache_ptr->pmpls) free(cache_ptr->pmpls);
     if (cache_ptr->pnat) free(cache_ptr->pnat);
+    if (cache_ptr->pmpls) free(cache_ptr->pmpls);
+    if (cache_ptr->ptun) free(cache_ptr->ptun);
     if (cache_ptr->pcust) free(cache_ptr->pcust);
     if (cache_ptr->pvlen) free(cache_ptr->pvlen);
     if (cache_ptr->stitch) free(cache_ptr->stitch);
@@ -565,8 +595,9 @@ void P_cache_insert_pending(struct chained_cache *queue[], int index, struct cha
     memcpy(cache_ptr, &container[j], dbc_size); 
 
     container[j].pbgp = NULL;
-    container[j].pmpls = NULL;
     container[j].pnat = NULL;
+    container[j].pmpls = NULL;
+    container[j].ptun = NULL;
     container[j].pcust = NULL;
     container[j].pvlen = NULL;
     container[j].stitch = NULL;
@@ -589,7 +620,7 @@ void P_cache_handle_flush_event(struct ports_table *pt)
     switch (ret = fork()) {
     case 0: /* Child */
       pm_setproctitle("%s %s [%s]", config.type, "Plugin -- Writer", config.name);
-      (*purge_func)(queries_queue, qq_ptr);
+      (*purge_func)(queries_queue, qq_ptr, FALSE);
       exit(0);
     default: /* Parent */
       if (ret == -1) Log(LOG_WARNING, "WARN ( %s/%s ): Unable to fork writer: %s\n", config.name, config.type, strerror(errno));
@@ -654,8 +685,9 @@ void P_cache_mark_flush(struct chained_cache *queue[], int index, int exiting)
       memcpy(&pqq_container[j], pending_queries_queue[j], dbc_size);
 
       pending_queries_queue[j]->pbgp = NULL;
-      pending_queries_queue[j]->pmpls = NULL;
       pending_queries_queue[j]->pnat = NULL;
+      pending_queries_queue[j]->pmpls = NULL;
+      pending_queries_queue[j]->ptun = NULL;
       pending_queries_queue[j]->pcust = NULL;
       pending_queries_queue[j]->pvlen = NULL;
       pending_queries_queue[j]->stitch = NULL;
@@ -748,9 +780,11 @@ void P_exit_now(int signum)
   if (qq_ptr) P_cache_mark_flush(queries_queue, qq_ptr, TRUE);
 
   dump_writers_count();
-  if (dump_writers_get_flags() != CHLD_ALERT) (*purge_func)(queries_queue, qq_ptr);
+  if (dump_writers_get_flags() != CHLD_ALERT) (*purge_func)(queries_queue, qq_ptr, FALSE);
   else Log(LOG_WARNING, "WARN ( %s/%s ): Maximum number of writer processes reached (%d).\n", config.name, config.type, dump_writers_get_active());
 
+  if (config.pidfile) remove_pid_file(config.pidfile);
+
   wait(NULL);
   exit_plugin(0);
 }
@@ -780,7 +814,8 @@ void P_init_historical_acct(time_t now)
   basetime.tv_sec = now;
   basetime.tv_usec = 0;
 
-  if (config.sql_history == COUNT_MINUTELY) timeslot = config.sql_history_howmany*60;
+  if (config.sql_history == COUNT_SECONDLY) timeslot = config.sql_history_howmany;
+  else if (config.sql_history == COUNT_MINUTELY) timeslot = config.sql_history_howmany*60;
   else if (config.sql_history == COUNT_HOURLY) timeslot = config.sql_history_howmany*3600;
   else if (config.sql_history == COUNT_DAILY) timeslot = config.sql_history_howmany*86400;
   else if (config.sql_history == COUNT_WEEKLY) timeslot = config.sql_history_howmany*86400*7;
@@ -811,14 +846,14 @@ void P_init_historical_acct(time_t now)
   memset(&new_basetime, 0, sizeof(new_basetime));
 }
 
-void P_init_refresh_deadline(time_t *rd)
+void P_init_refresh_deadline(time_t *now, int refresh_time, int startup_delay, char *roundoff)
 {
   time_t t;
 
-  t = roundoff_time(*rd, config.sql_history_roundoff);
-  while ((t+config.sql_refresh_time) < *rd) t += config.sql_refresh_time;
-  *rd = t;
-  *rd += (config.sql_refresh_time+config.sql_startup_delay); /* it's a deadline not a basetime */
+  t = roundoff_time((*now), roundoff);
+  while ((t + refresh_time) < (*now)) t += refresh_time;
+  *now = t;
+  *now += (refresh_time + startup_delay); /* it's a deadline not a basetime */
 }
 
 void P_eval_historical_acct(struct timeval *stamp, struct timeval *basetime, time_t timeslot)
@@ -890,6 +925,7 @@ void primptrs_set_all_from_chained_cache(struct primitives_ptrs *prim_ptrs, stru
     prim_ptrs->pbgp = entry->pbgp;
     prim_ptrs->pnat = entry->pnat;
     prim_ptrs->pmpls = entry->pmpls;
+    prim_ptrs->ptun = entry->ptun;
     prim_ptrs->pcust = entry->pcust;
     prim_ptrs->pvlen = entry->pvlen;
   }
@@ -995,6 +1031,20 @@ void P_handle_table_dyn_strings(char *new, int newlen, char *old, struct chained
   }
 }
 
+void P_update_time_reference(struct insert_data *idata)
+{
+  idata->now = time(NULL);
+
+  if (config.sql_history) {
+    while (idata->now > (basetime.tv_sec + timeslot)) {
+      new_basetime.tv_sec = basetime.tv_sec;
+      basetime.tv_sec += timeslot;
+      if (config.sql_history == COUNT_MONTHLY)
+	timeslot = calc_monthly_timeslot(basetime.tv_sec, config.sql_history_howmany, ADD);
+    }
+  }
+}
+
 void P_broker_timers_set_last_fail(struct p_broker_timers *btimers, time_t timestamp)
 {
   if (btimers) btimers->last_fail = timestamp;
