@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -26,6 +26,7 @@
 #include "pmacct-data.h"
 #include "thread_pool.h"
 #include "plugin_hooks.h"
+#include "plugin_common.h"
 #include "pkt_handlers.h"
 
 /* functions */
@@ -75,6 +76,7 @@ void load_plugins(struct plugin_requests *req)
       if (list->cfg.data_type & PIPE_TYPE_BGP) min_sz += sizeof(struct pkt_bgp_primitives);
       if (list->cfg.data_type & PIPE_TYPE_LBGP) min_sz += sizeof(struct pkt_legacy_bgp_primitives);
       if (list->cfg.data_type & PIPE_TYPE_NAT) min_sz += sizeof(struct pkt_nat_primitives);
+      if (list->cfg.data_type & PIPE_TYPE_TUN) min_sz += sizeof(struct pkt_tunnel_primitives);
       if (list->cfg.data_type & PIPE_TYPE_MPLS) min_sz += sizeof(struct pkt_mpls_primitives);
       if (list->cfg.cpptrs.len) min_sz += list->cfg.cpptrs.len;
       if (list->cfg.data_type & PIPE_TYPE_VLEN) {
@@ -104,7 +106,7 @@ void load_plugins(struct plugin_requests *req)
       while (list->cfg.buffer_size % 4 != 0) list->cfg.buffer_size--;
 #endif
 
-      if (!list->cfg.pipe_amqp) {
+      if (!list->cfg.pipe_zmq) {
         /* creating communication channel */
         socketpair(AF_UNIX, SOCK_DGRAM, 0, list->pipe);
 
@@ -178,31 +180,43 @@ void load_plugins(struct plugin_requests *req)
 	offset = sizeof(struct pkt_data);
       }
       if (list->cfg.data_type & PIPE_TYPE_PAYLOAD) chptr->clean_func = pkt_payload_clean;
+
       if (list->cfg.data_type & PIPE_TYPE_EXTRAS) {
 	chptr->extras.off_pkt_extras = offset;
 	offset += sizeof(struct pkt_extras);
       }
       if (list->cfg.data_type & PIPE_TYPE_MSG) chptr->clean_func = pkt_msg_clean;
+
       if (list->cfg.data_type & PIPE_TYPE_BGP) {
         chptr->extras.off_pkt_bgp_primitives = offset;
 	offset += sizeof(struct pkt_bgp_primitives);
       }
       else chptr->extras.off_pkt_bgp_primitives = 0; 
+
       if (list->cfg.data_type & PIPE_TYPE_LBGP) {
         chptr->extras.off_pkt_lbgp_primitives = offset;
         offset += sizeof(struct pkt_legacy_bgp_primitives);
       }
       else chptr->extras.off_pkt_lbgp_primitives = 0;
+
       if (list->cfg.data_type & PIPE_TYPE_NAT) {
         chptr->extras.off_pkt_nat_primitives = offset;
         offset += sizeof(struct pkt_nat_primitives);
       }
       else chptr->extras.off_pkt_nat_primitives = 0; 
+
+      if (list->cfg.data_type & PIPE_TYPE_TUN) {
+        chptr->extras.off_pkt_tun_primitives = offset;
+        offset += sizeof(struct pkt_tunnel_primitives);
+      }
+      else chptr->extras.off_pkt_tun_primitives = 0;
+
       if (list->cfg.data_type & PIPE_TYPE_MPLS) {
         chptr->extras.off_pkt_mpls_primitives = offset;
         offset += sizeof(struct pkt_mpls_primitives);
       }
       else chptr->extras.off_pkt_mpls_primitives = 0;
+
       if (list->cfg.cpptrs.len) {
 	chptr->extras.off_custom_primitives = offset;
 	offset += list->cfg.cpptrs.len;
@@ -226,6 +240,14 @@ void load_plugins(struct plugin_requests *req)
 	list->cfg.nfprobe_id = nfprobe_id;
 	nfprobe_id++;
       }
+
+      /* ZMQ inits, if required */
+#ifdef WITH_ZMQ
+      if (list->cfg.pipe_zmq) {
+	p_zmq_plugin_pipe_init_core(&chptr->zmq_host, list->id);
+	p_zmq_plugin_pipe_publish(&chptr->zmq_host);
+      }
+#endif
       
       switch (list->pid = fork()) {  
       case -1: /* Something went wrong */
@@ -247,11 +269,11 @@ void load_plugins(struct plugin_requests *req)
 
 	close(config.sock);
 	close(config.bgp_sock);
-	if (!list->cfg.pipe_amqp) close(list->pipe[1]);
+	if (!list->cfg.pipe_zmq) close(list->pipe[1]);
 	(*list->type.func)(list->pipe[0], &list->cfg, chptr);
 	exit(0);
       default: /* Parent */
-	if (!list->cfg.pipe_amqp) {
+	if (!list->cfg.pipe_zmq) {
 	  close(list->pipe[0]);
 	  setnonblocking(list->pipe[1]);
 	}
@@ -310,91 +332,6 @@ void load_plugins(struct plugin_requests *req)
       list = list->next;
     }
   }
-
-  /* AMQP handling, if required */
-#ifdef WITH_RABBITMQ
-  {
-    int ret, index, index2;
-
-    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
-      chptr = &channels_list[index];
-      list = chptr->plugin;
-
-      if (list->cfg.pipe_amqp) {
-        plugin_pipe_amqp_init_host(&chptr->amqp_host, list);
-        ret = p_amqp_connect_to_publish(&chptr->amqp_host);
-        if (ret) plugin_pipe_amqp_sleeper_start(chptr);
-      }
-
-      /* reset core process pipe AMQP routing key */
-      if (list->type.id == PLUGIN_ID_CORE) list->cfg.pipe_amqp_routing_key = NULL;
-    }
-
-    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
-      struct plugins_list_entry *list2 = plugins_list;
-      struct channels_list_entry *chptr2 = NULL;
-
-      chptr = &channels_list[index];
-      list = chptr->plugin;
-
-      for (index2 = index; channels_list[index2].aggregation || channels_list[index2].aggregation_2; index2++) {
-        chptr2 = &channels_list[index2];
-        list2 = chptr2->plugin;
-
-	if (index2 > index && list->cfg.pipe_amqp_exchange && list->cfg.pipe_amqp_routing_key) {
-	  if (!strcmp(list->cfg.pipe_amqp_exchange, list2->cfg.pipe_amqp_exchange) &&
-	      !strcmp(list->cfg.pipe_amqp_routing_key, list2->cfg.pipe_amqp_routing_key)) {
-	    Log(LOG_ERR, "ERROR ( %s/%s ): Duplicated plugin_pipe_amqp_exchange, plugin_pipe_amqp_routing_key: %s, %s\nExiting.\n",
-		list->name, list->type.string, list->cfg.pipe_amqp_exchange, list->cfg.pipe_amqp_routing_key);
-	    exit(1);
-	  }
-        }
-      }
-    }
-  }
-#endif
-
-  /* Kafka handling, if required */
-#ifdef WITH_KAFKA
-  {
-    int ret, index, index2;
-
-    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
-      chptr = &channels_list[index];
-      list = chptr->plugin;
-
-      /* XXX: no sleeper thread, trusting librdkafka */
-      if (list->cfg.pipe_kafka) ret = plugin_pipe_kafka_init_host(&chptr->kafka_host, list, TRUE);
-
-      /* reset core process pipe Kafka topic */
-      if (list->type.id == PLUGIN_ID_CORE) list->cfg.pipe_kafka_topic = NULL;
-    }
-
-    for (index = 0; channels_list[index].aggregation || channels_list[index].aggregation_2; index++) {
-      struct plugins_list_entry *list2 = plugins_list;
-      struct channels_list_entry *chptr2 = NULL;
-
-      chptr = &channels_list[index];
-      list = chptr->plugin;
-
-      for (index2 = index; channels_list[index2].aggregation || channels_list[index2].aggregation_2; index2++) {
-        chptr2 = &channels_list[index2];
-        list2 = chptr2->plugin;
-
-        if (index2 > index && list->cfg.pipe_kafka_broker_host && list->cfg.pipe_kafka_topic) {
-          if (!strcmp(list->cfg.pipe_kafka_broker_host, list2->cfg.pipe_kafka_broker_host) &&
-              list->cfg.pipe_kafka_broker_port == list2->cfg.pipe_kafka_broker_port &&
-              !strcmp(list->cfg.pipe_kafka_topic, list2->cfg.pipe_kafka_topic) /* && XXX: topic partition too? */ ) {
-            Log(LOG_ERR, "ERROR ( %s/%s ): Duplicated plugin_pipe_kafka_broker_*, plugin_pipe_kafka_topic: %s, %s, %s\nExiting.\n",
-                list->name, list->type.string, list->cfg.pipe_kafka_broker_host, list->cfg.pipe_kafka_broker_port,
-		list->cfg.pipe_kafka_topic);
-            exit(1);
-          }
-        }
-      }
-    }
-  }
-#endif
 }
 
 void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req) 
@@ -426,7 +363,8 @@ void exec_plugins(struct packet_ptrs *pptrs, struct plugin_requests *req)
 
     if (p->cfg.pre_tag_map && find_id_func) {
       if (p->cfg.type_id == PLUGIN_ID_TEE) {
-	if ((req->ptm_c.exec_ptm_res && !p->cfg.ptm_complex) || (!req->ptm_c.exec_ptm_res && p->cfg.ptm_complex)) 
+	if ((req->ptm_c.exec_ptm_res && !p->cfg.ptm_complex) ||
+	    ((!req->ptm_c.exec_ptm_res && p->cfg.ptm_complex) && !p->cfg.tee_dissect_send_full_pkt)) 
 	  continue;
       }
 
@@ -526,39 +464,20 @@ reprocess:
 		channels_list[index].hdr.seq, channels_list[index].hdr.num, channels_list[index].status->last_buf_off);
 	}
 
-	/* sending the buffer to the AMQP broker */
-	if (channels_list[index].plugin->cfg.pipe_amqp) {
-#ifdef WITH_RABBITMQ
+	/* sending buffer to connected ZMQ subscriber(s) */
+	if (channels_list[index].plugin->cfg.pipe_zmq) {
+#ifdef WITH_ZMQ
           struct channels_list_entry *chptr = &channels_list[index];
 
-          plugin_pipe_amqp_sleeper_stop(chptr);
-	  if (!chptr->amqp_host_sleep) ret = p_amqp_publish_binary(&chptr->amqp_host, chptr->rg.ptr, chptr->bufsize);
-	  else ret = FALSE;
-          if (ret) plugin_pipe_amqp_sleeper_start(chptr);
-#endif
-	}
-	/* sending the buffer to the Kafka broker */
-	else if (channels_list[index].plugin->cfg.pipe_kafka) {
-#ifdef WITH_KAFKA
-          struct channels_list_entry *chptr = &channels_list[index];
-
-	  /* XXX: no sleeper thread, trusting librdkafka */
-	  ret = p_kafka_produce_data(&chptr->kafka_host, chptr->rg.ptr, chptr->bufsize);
+	  ret = p_zmq_plugin_pipe_send(&chptr->zmq_host, chptr->rg.ptr, chptr->bufsize);
 #endif
 	}
 	else {
 	  if (channels_list[index].status->wakeup) {
-	    channels_list[index].status->backlog++;
-	  
-	    if (channels_list[index].status->backlog >
-		((channels_list[index].plugin->cfg.pipe_size/channels_list[index].plugin->cfg.buffer_size)
-		*channels_list[index].plugin->cfg.pipe_backlog)/100) {
-	      channels_list[index].status->wakeup = channels_list[index].request;
-              if (write(channels_list[index].pipe, &channels_list[index].rg.ptr, CharPtrSz) != CharPtrSz) {
-	        struct plugins_list_entry *list = channels_list[index].plugin;
-	        Log(LOG_WARNING, "WARN ( %s/%s ): Failed during write: %s\n", list->name, list->type.string, strerror(errno));
-	      }
-	      channels_list[index].status->backlog = 0;
+	    channels_list[index].status->wakeup = channels_list[index].request;
+	    if (write(channels_list[index].pipe, &channels_list[index].rg.ptr, CharPtrSz) != CharPtrSz) {
+	      struct plugins_list_entry *list = channels_list[index].plugin;
+	      Log(LOG_WARNING, "WARN ( %s/%s ): Failed during write: %s\n", list->name, list->type.string, strerror(errno));
 	    }
 	  }
 	}
@@ -872,14 +791,9 @@ void fill_pipe_buffer()
     ((struct ch_buf_hdr *)chptr->rg.ptr)->num = chptr->hdr.num;
     ((struct ch_buf_hdr *)chptr->rg.ptr)->core_pid = chptr->core_pid;
 
-    if (chptr->plugin->cfg.pipe_amqp) {
-#ifdef WITH_RABBITMQ
-      p_amqp_publish_binary(&chptr->amqp_host, chptr->rg.ptr, chptr->bufsize);
-#endif
-    }
-    else if (chptr->plugin->cfg.pipe_kafka) {
-#ifdef WITH_KAFKA
-      p_kafka_produce_data(&chptr->kafka_host, chptr->rg.ptr, chptr->bufsize);
+    if (chptr->plugin->cfg.pipe_zmq) {
+#ifdef WITH_ZMQ
+      p_zmq_plugin_pipe_send(&chptr->zmq_host, chptr->rg.ptr, chptr->bufsize);
 #endif
     }
     else {
@@ -1004,243 +918,15 @@ int pkt_extras_clean(void *pextras, int len)
   return PdataSz+PextrasSz;
 }
 
-void handle_plugin_pipe_dyn_strings(char *new, int newlen, char *old, struct plugins_list_entry *list)
+void plugin_pipe_zmq_compile_check()
 {
-  int oldlen, ptr_len;
-  char core_proc_name[] = "$core_proc_name", plugin_name[] = "$plugin_name";
-  char plugin_type[] = "$plugin_type";
-  char *ptr_start, *ptr_end;
-
-  if (!new || !old || !list) return;
-
-  oldlen = strlen(old);
-  if (oldlen <= newlen) strcpy(new, old);
-  else {
-    strncpy(new, old, newlen);
-    return;
-  }
-
-  replace_string(new, newlen, core_proc_name, list->cfg.proc_name);
-  replace_string(new, newlen, plugin_name, list->cfg.name);
-  replace_string(new, newlen, plugin_type, list->cfg.type);
-}
-
-char *plugin_pipe_compose_default_string(struct plugins_list_entry *list, char *default_rk)
-{
-  char *rk = NULL;
-
-  if (!list || !default_rk) return rk;
-
-  rk = malloc(SRVBUFLEN);
-  memset(rk, 0, SRVBUFLEN);
-
-  handle_plugin_pipe_dyn_strings(rk, SRVBUFLEN, default_rk, list);
-
-  return rk;
-}
-
-#ifdef WITH_RABBITMQ
-void plugin_pipe_amqp_init_host(struct p_amqp_host *amqp_host, struct plugins_list_entry *list)
-{
-  int ret;
-
-  if (amqp_host) {
-    char *amqp_rk = plugin_pipe_compose_default_string(list, "$core_proc_name-$plugin_name-$plugin_type");
-
-    p_amqp_init_host(amqp_host);
-
-    if (!list->cfg.pipe_amqp_user) list->cfg.pipe_amqp_user = rabbitmq_user;
-    if (!list->cfg.pipe_amqp_passwd) list->cfg.pipe_amqp_passwd = rabbitmq_pwd;
-    if (!list->cfg.pipe_amqp_exchange) list->cfg.pipe_amqp_exchange = default_amqp_exchange;
-    if (!list->cfg.pipe_amqp_host) list->cfg.pipe_amqp_host = default_amqp_host;
-    if (!list->cfg.pipe_amqp_vhost) list->cfg.pipe_amqp_vhost = default_amqp_vhost;
-    if (!list->cfg.pipe_amqp_routing_key) list->cfg.pipe_amqp_routing_key = amqp_rk;
-    if (!list->cfg.pipe_amqp_retry) list->cfg.pipe_amqp_retry = AMQP_DEFAULT_RETRY;
-
-    p_amqp_set_user(amqp_host, list->cfg.pipe_amqp_user);
-    p_amqp_set_passwd(amqp_host, list->cfg.pipe_amqp_passwd);
-    p_amqp_set_exchange(amqp_host, list->cfg.pipe_amqp_exchange);
-    p_amqp_set_host(amqp_host, list->cfg.pipe_amqp_host);
-    p_amqp_set_vhost(amqp_host, list->cfg.pipe_amqp_vhost);
-    p_amqp_set_routing_key(amqp_host, list->cfg.pipe_amqp_routing_key);
-    P_broker_timers_set_retry_interval(&amqp_host->btimers, list->cfg.pipe_amqp_retry);
-
-    p_amqp_set_frame_max(amqp_host, list->cfg.buffer_size);
-    p_amqp_set_exchange_type(amqp_host, default_amqp_exchange_type);
-    p_amqp_set_content_type_binary(amqp_host);
-  }
-}
-
-struct plugin_pipe_amqp_sleeper *plugin_pipe_amqp_sleeper_define(struct p_amqp_host *amqp_host, int *flag, struct plugins_list_entry *plugin)
-{
-  struct plugin_pipe_amqp_sleeper *pas;
-  int size = sizeof(struct plugin_pipe_amqp_sleeper);
-
-  if (!amqp_host || !flag) return NULL;
-
-  pas = malloc(size);
-
-  if (pas) {
-    memset(pas, 0, size);
-    pas->amqp_host = amqp_host;
-    pas->plugin = plugin;
-    pas->do_reconnect = flag;
-  }
-  else {
-    Log(LOG_ERR, "ERROR ( %s/%s ): plugin_pipe_amqp_sleeper_define(): malloc() failed\n", plugin->cfg.name, plugin->cfg.type);
-    return NULL;
-  }
-
-  return pas;
-}
-
-void plugin_pipe_amqp_sleeper_free(struct plugin_pipe_amqp_sleeper **pas)
-{
-  if (!pas || !(*pas)) return;
-
-  free((*pas));
-  (*pas) = NULL;
-}
-
-void plugin_pipe_amqp_sleeper_publish_func(struct plugin_pipe_amqp_sleeper *pas)
-{
-  int ret;
-
-  if (!pas || !pas->amqp_host || !pas->plugin || !pas->do_reconnect) return;
-
-sleep_again:
-  sleep(P_broker_timers_get_retry_interval(&pas->amqp_host->btimers));
-
-  plugin_pipe_amqp_init_host(pas->amqp_host, pas->plugin);
-  ret = p_amqp_connect_to_publish(pas->amqp_host);
-
-  if (ret) goto sleep_again;
-
-  (*pas->do_reconnect) = TRUE;
-
-  plugin_pipe_amqp_sleeper_free(&pas);
-}
-
-void plugin_pipe_amqp_sleeper_start(struct channels_list_entry *chptr)
-{
-#if defined ENABLE_THREADS
-  if (chptr && !chptr->amqp_host_sleep) {
-    struct plugin_pipe_amqp_sleeper *pas;
-
-    chptr->amqp_host_sleep = allocate_thread_pool(1);
-    assert(chptr->amqp_host_sleep);
-
-    pas = plugin_pipe_amqp_sleeper_define(&chptr->amqp_host, &chptr->amqp_host_reconnect, chptr->plugin);
-    if (pas) send_to_pool((thread_pool_t *) chptr->amqp_host_sleep, plugin_pipe_amqp_sleeper_publish_func, pas);
-    else Log(LOG_ERR, "ERROR ( %s/%s ): plugin_pipe_amqp_sleeper_start(): sleeper define failed\n", chptr->plugin->cfg.name, chptr->plugin->cfg.type);
-  }
-#endif
-}
-
-void plugin_pipe_amqp_sleeper_stop(struct channels_list_entry *chptr)
-{
-#if defined ENABLE_THREADS
-  if (chptr && chptr->amqp_host_reconnect) {
-    deallocate_thread_pool((thread_pool_t **) &chptr->amqp_host_sleep);
-    chptr->amqp_host_sleep = NULL;
-    chptr->amqp_host_reconnect = FALSE;
-  }
-#endif
-}
-
-int plugin_pipe_amqp_connect_to_consume(struct p_amqp_host *amqp_host, struct plugins_list_entry *plugin_data)
-{
-  plugin_pipe_amqp_init_host(amqp_host, plugin_data);
-  p_amqp_connect_to_consume(amqp_host);
-  return p_amqp_get_sockfd(amqp_host);
-}
-#endif
-
-#if defined WITH_KAFKA
-int plugin_pipe_kafka_init_host(struct p_kafka_host *kafka_host, struct plugins_list_entry *list, int is_prod)
-{
-  int ret = SUCCESS;
-
-  if (kafka_host && list && !validate_truefalse(is_prod)) {
-    char *topic = plugin_pipe_compose_default_string(list, "pmacct.$core_proc_name-$plugin_name-$plugin_type");
-
-    p_kafka_init_host(kafka_host);
-
-    if (is_prod) ret = p_kafka_connect_to_produce(kafka_host);
-    else ret = p_kafka_connect_to_consume(kafka_host);
-
-    if (!list->cfg.pipe_kafka_broker_host) list->cfg.pipe_kafka_broker_host = default_kafka_broker_host;
-    if (!list->cfg.pipe_kafka_broker_port) list->cfg.pipe_kafka_broker_port = default_kafka_broker_port;
-    if (!list->cfg.pipe_kafka_topic) list->cfg.pipe_kafka_topic = topic;
-    if (!list->cfg.pipe_kafka_retry) list->cfg.pipe_kafka_retry = PM_KAFKA_DEFAULT_RETRY;
-
-    p_kafka_set_broker(kafka_host, list->cfg.pipe_kafka_broker_host, list->cfg.pipe_kafka_broker_port);
-    p_kafka_set_topic(kafka_host, list->cfg.pipe_kafka_topic);
-    p_kafka_set_partition(kafka_host, list->cfg.pipe_kafka_partition);
-    p_kafka_set_key(kafka_host, list->cfg.pipe_kafka_partition_key, list->cfg.pipe_kafka_partition_keylen);
-    p_kafka_set_content_type(kafka_host, PM_KAFKA_CNT_TYPE_BIN);
-    P_broker_timers_set_retry_interval(&kafka_host->btimers, list->cfg.pipe_kafka_retry);
-  }
-  else return ERR;
-
-  return ret;
-}
-
-int plugin_pipe_kafka_connect_to_consume(struct p_kafka_host *kafka_host, struct plugins_list_entry *plugin_data)
-{
-  int ret = SUCCESS;
-
-  if (kafka_host && plugin_data) {
-    ret = plugin_pipe_kafka_init_host(kafka_host, plugin_data, FALSE);
-    if (!ret) ret = p_kafka_manage_consumer(kafka_host, TRUE);
-  }
-  else return ERR;
-
-  return ret;
-}
-#endif 
-
-int plugin_pipe_set_retry_timeout(struct p_broker_timers *btimers, int pipe_fd)
-{
-  if (pipe_fd == ERR) return (P_broker_timers_get_retry_interval(btimers) * 1000);
-  else return LONGLONG_RETRY;
-}
-
-int plugin_pipe_calc_retry_timeout_diff(struct p_broker_timers *btimers, time_t now)
-{
-  int timeout;
-
-  timeout = (((P_broker_timers_get_last_fail(btimers) + P_broker_timers_get_retry_interval(btimers)) - now) * 1000);
-  assert(timeout >= 0);
-
-  return timeout;
-}
-
-void plugin_pipe_amqp_compile_check()
-{
-#ifndef WITH_RABBITMQ
-  Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_amqp' requires compiling with --enable-rabbitmq. Exiting ..\n", config.name, config.type);
-  exit_plugin(1);
-#endif
-}
-
-void plugin_pipe_kafka_compile_check()
-{
-#ifndef WITH_KAFKA
-  Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_kafka' requires compiling with --enable-kafka. Exiting ..\n", config.name, config.type);
+#ifndef WITH_ZMQ
+  Log(LOG_ERR, "ERROR ( %s/%s ): 'plugin_pipe_zmq' requires compiling with --enable-zmq. Exiting ..\n", config.name, config.type);
   exit_plugin(1);
 #endif
 }
 
 void plugin_pipe_check(struct configuration *cfg)
 {
-  if (!cfg->pipe_amqp && !cfg->pipe_kafka) cfg->pipe_homegrown = TRUE;
-
-  if (cfg->pipe_amqp && cfg->pipe_kafka) {
-    Log(LOG_WARNING, "WARN ( %s/%s ): 'plugin_pipe_amqp' and 'plugin_pipe_kafka' are mutual exclusive: disabling both.\n", cfg->name, cfg->type);
-
-    cfg->pipe_amqp = FALSE;
-    cfg->pipe_kafka = FALSE;
-    cfg->pipe_homegrown = TRUE;
-  }
+  if (!cfg->pipe_zmq) cfg->pipe_homegrown = TRUE;
 }
