@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -71,11 +71,13 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"as_path", 1, 1, 1, 1, 0, 0, 0, "AS PATH"},
   {"std_comm", 1, 1, 1, 1, 0, 0, 0, "Standard Communities"},
   {"ext_comm", 1, 1, 1, 1, 0, 0, 0, "Extended Communities"},
+  {"lrg_comm", 1, 1, 1, 1, 0, 0, 0, "Large Communities"},
   {"local_pref", 1, 1, 1, 1, 0, 0, 0, "Local Preference"},
   {"med", 1, 1, 1, 1, 0, 0, 0, "Multi-Exit Discriminator"},
   {"src_as_path", 1, 1, 1, 1, 0, 0, 0, "Source AS PATH (via reverse BGP lookup)"},
   {"src_std_comm", 1, 1, 1, 1, 0, 0, 0, "Source Standard Communities (via reverse BGP lookup)"},
   {"src_ext_comm", 1, 1, 1, 1, 0, 0, 0, "Source Extended Communities (via reverse BGP lookup)"},
+  {"src_lrg_comm", 1, 1, 1, 1, 0, 0, 0, "Source Large Communities (via reverse BGP lookup)"},
   {"src_local_pref", 1, 1, 1, 1, 0, 0, 0, "Source Local Preference (by default via reverse BGP lookup)"},
   {"src_med", 1, 1, 1, 1, 0, 0, 0, "Source MED (by default via reverse BGP lookup)"},
   {"mpls_vpn_rd", 0, 0, 1, 1, 0, 0, 0, "MPLS VPN Route Distinguisher"},
@@ -89,6 +91,11 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"post_nat_dst_host", 0, 0, 1, 0, 0, 0, 0, "Destination IPv4/IPv6 address after NAT translation"},
   {"post_nat_src_port", 0, 0, 1, 0, 0, 0, 0, "Source TCP/UDP port after NAT translation"},
   {"post_nat_dst_port", 0, 0, 1, 0, 0, 0, 0, "Destination TCP/UDP port after NAT translation"},
+  {"TUNNEL", 1, 1, 1, 1, 0, 0, 0, ""}, 
+  {"tunnel_src_host", 0, 0, 0, 1, 0, 0, 0, "Tunnel inner Source IPv4/IPv6 address"},
+  {"tunnel_dst_host", 0, 0, 0, 1, 0, 0, 0, "Tunnel inner Destination IPv4/IPv6 address"},
+  {"tunnel_proto", 0, 0, 0, 1, 0, 0, 0, "Tunnel inner IP protocol"},
+  {"tunnel_tos", 0, 0, 0, 1, 0, 0, 0, "Tunnel inner IP ToS"},
   {"MPLS", 1, 1, 1, 1, 0, 0, 0, ""}, 
   {"mpls_label_bottom", 1, 1, 1, 0, 0, 0, 0, "Bottom MPLS label"},
   {"mpls_label_top", 1, 1, 1, 0, 0, 0, 0, "Top MPLS label"},
@@ -98,6 +105,8 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
   {"flows", 0, 0, 1, 0, 0, 0, 0, "IP flows"},
   {"src_host_country", 1, 1, 1, 1, 0, 0, 0, "Source IP address GeoIP resolution: country"},
   {"dst_host_country", 1, 1, 1, 1, 0, 0, 0, "Destination IP address GeoIP resolution: country"},
+  {"src_host_pocode", 1, 1, 1, 1, 0, 0, 0, "Source IP address GeoIP resolution: postal code"},
+  {"dst_host_pocode", 1, 1, 1, 1, 0, 0, 0, "Destination IP address GeoIP resolution: postal code"},
   {"in_iface", 0, 1, 1, 1, 0, 0, 0, "Input interface, SNMP ifIndex"}, 
   {"out_iface", 0, 1, 1, 1, 0, 0, 0, "Output interface, SNMP ifIndex"}, 
   {"peer_src_ip", 0, 0, 1, 1, 0, 0, 0, "IP address or identificator of telemetry exporting device"},
@@ -116,11 +125,11 @@ static const struct _primitives_matrix_struct _primitives_matrix[] = {
 };
 
 static const struct _protocols_struct _protocols[] = {
-  {"ip", 0},
+  {"0", 0},
   {"icmp", 1},
   {"igmp", 2},
   {"ggp", 3},
-  {"4", 4},
+  {"ipencap", 4},
   {"5", 5},
   {"tcp", 6},
   {"7", 7},
@@ -276,7 +285,7 @@ static const int cps_flen[] = {
   20
 };
 
-#if defined __PMACCTD_C || defined __UACCTD_C
+#if defined __PMACCTD_C || defined __UACCTD_C || defined __UTIL_C
 static struct _devices_struct _devices[] = {
 #if defined DLT_LOOP
   {null_handler, DLT_LOOP},
@@ -284,8 +293,6 @@ static struct _devices_struct _devices[] = {
   {null_handler, DLT_NULL},
   {eth_handler, DLT_EN10MB},
   {ppp_handler, DLT_PPP},
-  {fddi_handler, DLT_FDDI},
-  {tr_handler, DLT_IEEE802},
 #if defined DLT_IEEE802_11
   {ieee_802_11_handler, DLT_IEEE802_11}, 
 #endif
@@ -294,12 +301,6 @@ static struct _devices_struct _devices[] = {
 #endif
 #if defined DLT_RAW
   {raw_handler, DLT_RAW},
-#endif
-#if defined DLT_C_HDLC
-  {chdlc_handler, DLT_C_HDLC},
-#endif
-#ifdef DLT_HDLC
-  {chdlc_handler, DLT_HDLC},
 #endif
   {NULL, -1},
 };
@@ -319,6 +320,9 @@ static const struct _dictionary_line dictionary[] = {
   {"aggregate_filter", cfg_key_aggregate_filter},
   {"promisc", cfg_key_promisc},
   {"pcap_filter", cfg_key_pcap_filter},
+  {"pcap_protocol", cfg_key_pcap_protocol},
+  {"pcap_savefile", cfg_key_pcap_savefile},
+  {"pcap_savefile_wait", cfg_key_pcap_savefile_wait},
   {"core_proc_name", cfg_key_proc_name},
   {"proc_priority", cfg_key_proc_priority},
   {"pmacctd_as", cfg_key_nfacctd_as_new},
@@ -329,33 +333,22 @@ static const struct _dictionary_line dictionary[] = {
   {"thread_stack", cfg_key_thread_stack},
   {"plugins", NULL},
   {"plugin_pipe_size", cfg_key_plugin_pipe_size},
-  {"plugin_pipe_backlog", cfg_key_plugin_pipe_backlog},
-  {"plugin_pipe_check_core_pid", cfg_key_plugin_pipe_check_core_pid},
-  {"plugin_pipe_amqp", cfg_key_plugin_pipe_amqp},
-  {"plugin_pipe_amqp_user", cfg_key_plugin_pipe_amqp_user},
-  {"plugin_pipe_amqp_passwd", cfg_key_plugin_pipe_amqp_passwd},
-  {"plugin_pipe_amqp_exchange", cfg_key_plugin_pipe_amqp_exchange},
-  {"plugin_pipe_amqp_host", cfg_key_plugin_pipe_amqp_host},
-  {"plugin_pipe_amqp_vhost", cfg_key_plugin_pipe_amqp_vhost},
-  {"plugin_pipe_amqp_routing_key", cfg_key_plugin_pipe_amqp_routing_key},
-  {"plugin_pipe_amqp_retry", cfg_key_plugin_pipe_amqp_retry},
-  {"plugin_pipe_kafka", cfg_key_plugin_pipe_kafka},
-  {"plugin_pipe_kafka_broker_host", cfg_key_plugin_pipe_kafka_broker_host},
-  {"plugin_pipe_kafka_broker_port", cfg_key_plugin_pipe_kafka_broker_port},
-  {"plugin_pipe_kafka_topic", cfg_key_plugin_pipe_kafka_topic},
-  {"plugin_pipe_kafka_partition", cfg_key_plugin_pipe_kafka_partition},
-  {"plugin_pipe_kafka_retry", cfg_key_plugin_pipe_kafka_retry},
   {"plugin_buffer_size", cfg_key_plugin_buffer_size},
+  {"plugin_pipe_check_core_pid", cfg_key_plugin_pipe_check_core_pid},
+  {"plugin_pipe_zmq", cfg_key_plugin_pipe_zmq},
+  {"plugin_pipe_zmq_retry", cfg_key_plugin_pipe_zmq_retry},
+  {"plugin_pipe_zmq_profile", cfg_key_plugin_pipe_zmq_profile},
   {"interface", cfg_key_interface},
   {"interface_wait", cfg_key_interface_wait},
   {"files_umask", cfg_key_files_umask},
   {"files_uid", cfg_key_files_uid},
   {"files_gid", cfg_key_files_gid},
-  {"savefile_wait", cfg_key_savefile_wait},
+  {"savefile_wait", cfg_key_pcap_savefile_wait}, /* XXX: legacy; to be obsoleted */
   {"networks_mask", cfg_key_networks_mask},
   {"networks_file", cfg_key_networks_file},
   {"networks_file_filter", cfg_key_networks_file_filter},
   {"networks_file_no_lpm", cfg_key_networks_file_no_lpm},
+  {"networks_no_mask_if_zero", cfg_key_networks_no_mask_if_zero},
   {"networks_cache_entries", cfg_key_networks_cache_entries},
   {"ports_file", cfg_key_ports_file},
   {"timestamps_secs", cfg_key_timestamps_secs},
@@ -384,7 +377,7 @@ static const struct _dictionary_line dictionary[] = {
   {"sql_history_since_epoch", cfg_key_timestamps_since_epoch}, // legacy
   {"sql_recovery_backup_host", cfg_key_sql_recovery_backup_host},
   {"sql_delimiter", cfg_key_sql_delimiter},
-  {"sql_max_writers", cfg_key_sql_max_writers},
+  {"sql_max_writers", cfg_key_dump_max_writers},
   {"sql_trigger_exec", cfg_key_sql_trigger_exec},
   {"sql_trigger_time", cfg_key_sql_trigger_time},
   {"sql_cache_entries", cfg_key_sql_cache_entries},
@@ -392,7 +385,6 @@ static const struct _dictionary_line dictionary[] = {
   {"sql_preprocess", cfg_key_sql_preprocess},
   {"sql_preprocess_type", cfg_key_sql_preprocess_type},
   {"sql_multi_values", cfg_key_sql_multi_values},
-  {"sql_aggressive_classification", cfg_key_sql_aggressive_classification},
   {"sql_locking_style", cfg_key_sql_locking_style},
   {"sql_use_copy", cfg_key_sql_use_copy},
   {"sql_num_protos", cfg_key_num_protos},
@@ -407,12 +399,11 @@ static const struct _dictionary_line dictionary[] = {
   {"print_output_separator", cfg_key_print_output_separator},
   {"print_latest_file", cfg_key_print_latest_file},
   {"print_num_protos", cfg_key_num_protos},
-  {"print_time_roundoff", cfg_key_sql_history_roundoff},
   {"print_trigger_exec", cfg_key_sql_trigger_exec},
   {"print_history", cfg_key_sql_history},
   {"print_history_offset", cfg_key_sql_history_offset},
   {"print_history_roundoff", cfg_key_sql_history_roundoff},
-  {"print_max_writers", cfg_key_sql_max_writers},
+  {"print_max_writers", cfg_key_dump_max_writers},
   {"print_preprocess", cfg_key_sql_preprocess},
   {"print_preprocess_type", cfg_key_sql_preprocess_type},
   {"print_startup_delay", cfg_key_sql_startup_delay},
@@ -429,7 +420,7 @@ static const struct _dictionary_line dictionary[] = {
   {"mongo_trigger_exec", cfg_key_sql_trigger_exec},
   {"mongo_insert_batch", cfg_key_mongo_insert_batch},
   {"mongo_indexes_file", cfg_key_sql_table_schema},
-  {"mongo_max_writers", cfg_key_sql_max_writers},
+  {"mongo_max_writers", cfg_key_dump_max_writers},
   {"mongo_preprocess", cfg_key_sql_preprocess},
   {"mongo_preprocess_type", cfg_key_sql_preprocess_type},
   {"mongo_startup_delay", cfg_key_sql_startup_delay},
@@ -451,7 +442,7 @@ static const struct _dictionary_line dictionary[] = {
   {"amqp_persistent_msg", cfg_key_amqp_persistent_msg},
   {"amqp_frame_max", cfg_key_amqp_frame_max},
   {"amqp_cache_entries", cfg_key_print_cache_entries},
-  {"amqp_max_writers", cfg_key_sql_max_writers},
+  {"amqp_max_writers", cfg_key_dump_max_writers},
   {"amqp_preprocess", cfg_key_sql_preprocess},
   {"amqp_preprocess_type", cfg_key_sql_preprocess_type},
   {"amqp_startup_delay", cfg_key_sql_startup_delay},
@@ -461,6 +452,9 @@ static const struct _dictionary_line dictionary[] = {
   {"amqp_vhost", cfg_key_amqp_vhost},
   {"amqp_markers", cfg_key_print_markers},
   {"amqp_output", cfg_key_message_broker_output},
+  {"amqp_avro_schema_routing_key", cfg_key_amqp_avro_schema_routing_key},
+  {"amqp_avro_schema_refresh_time", cfg_key_amqp_avro_schema_refresh_time},
+  {"amqp_trigger_exec", cfg_key_sql_trigger_exec},
   {"kafka_refresh_time", cfg_key_sql_refresh_time},
   {"kafka_history", cfg_key_sql_history},
   {"kafka_history_offset", cfg_key_sql_history_offset},
@@ -472,7 +466,7 @@ static const struct _dictionary_line dictionary[] = {
   {"kafka_partition", cfg_key_kafka_partition},
   {"kafka_partition_key", cfg_key_kafka_partition_key},
   {"kafka_cache_entries", cfg_key_print_cache_entries},
-  {"kafka_max_writers", cfg_key_sql_max_writers},
+  {"kafka_max_writers", cfg_key_dump_max_writers},
   {"kafka_preprocess", cfg_key_sql_preprocess},
   {"kafka_preprocess_type", cfg_key_sql_preprocess_type},
   {"kafka_startup_delay", cfg_key_sql_startup_delay},
@@ -480,6 +474,10 @@ static const struct _dictionary_line dictionary[] = {
   {"kafka_num_protos", cfg_key_num_protos},
   {"kafka_markers", cfg_key_print_markers},
   {"kafka_output", cfg_key_message_broker_output},
+  {"kafka_avro_schema_topic", cfg_key_kafka_avro_schema_topic},
+  {"kafka_avro_schema_refresh_time", cfg_key_kafka_avro_schema_refresh_time},
+  {"kafka_config_file", cfg_key_kafka_config_file},
+  {"kafka_trigger_exec", cfg_key_sql_trigger_exec},
   {"nfacctd_proc_name", cfg_key_proc_name},
   {"nfacctd_port", cfg_key_nfacctd_port},
   {"nfacctd_ip", cfg_key_nfacctd_ip},
@@ -493,11 +491,13 @@ static const struct _dictionary_line dictionary[] = {
   {"nfacctd_peer_as", cfg_key_nfprobe_peer_as},
   {"nfacctd_pipe_size", cfg_key_nfacctd_pipe_size},
   {"nfacctd_pro_rating", cfg_key_nfacctd_pro_rating},
+  {"nfacctd_templates_file", cfg_key_nfacctd_templates_file},
   {"nfacctd_account_options", cfg_key_nfacctd_account_options},
   {"nfacctd_stitching", cfg_key_nfacctd_stitching},
   {"nfacctd_ext_sampling_rate", cfg_key_pmacctd_ext_sampling_rate},
   {"nfacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"nfacctd_disable_checks", cfg_key_nfacctd_disable_checks},
+  {"nfacctd_disable_opt_scope_check", cfg_key_nfacctd_disable_opt_scope_check},
   {"pmacctd_proc_name", cfg_key_proc_name},
   {"pmacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
   {"pmacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
@@ -511,6 +511,7 @@ static const struct _dictionary_line dictionary[] = {
   {"pmacctd_stitching", cfg_key_nfacctd_stitching},
   {"pmacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"pmacctd_nonroot", cfg_key_pmacctd_nonroot},
+  {"pmacctd_time_new", cfg_key_nfacctd_time_new},
   {"uacctd_proc_name", cfg_key_proc_name},
   {"uacctd_force_frag_handling", cfg_key_pmacctd_force_frag_handling},
   {"uacctd_frag_buffer_size", cfg_key_pmacctd_frag_buffer_size},
@@ -553,6 +554,7 @@ static const struct _dictionary_line dictionary[] = {
   {"telemetry_daemon_msglog_kafka_partition", cfg_key_telemetry_msglog_kafka_partition},
   {"telemetry_daemon_msglog_kafka_partition_key", cfg_key_telemetry_msglog_kafka_partition_key},
   {"telemetry_daemon_msglog_kafka_retry", cfg_key_telemetry_msglog_kafka_retry},
+  {"telemetry_daemon_msglog_kafka_config_file", cfg_key_telemetry_msglog_kafka_config_file},
   {"telemetry_dump_output", cfg_key_telemetry_dump_output},
   {"telemetry_dump_file", cfg_key_telemetry_dump_file},
   {"telemetry_dump_latest_file", cfg_key_telemetry_dump_latest_file},
@@ -574,7 +576,7 @@ static const struct _dictionary_line dictionary[] = {
   {"telemetry_dump_kafka_topic_rr", cfg_key_telemetry_dump_kafka_topic_rr},
   {"telemetry_dump_kafka_partition", cfg_key_telemetry_dump_kafka_partition},
   {"telemetry_dump_kafka_partition_key", cfg_key_telemetry_dump_kafka_partition_key},
-  {"pcap_savefile", cfg_key_pcap_savefile},
+  {"telemetry_dump_kafka_config_file", cfg_key_telemetry_dump_kafka_config_file},
   {"refresh_maps", cfg_key_maps_refresh}, // legacy
   {"maps_refresh", cfg_key_maps_refresh},
   {"maps_index", cfg_key_maps_index},
@@ -597,6 +599,7 @@ static const struct _dictionary_line dictionary[] = {
   {"sfacctd_as", cfg_key_nfacctd_as_new},
   {"sfacctd_net", cfg_key_nfacctd_net},
   {"sfacctd_peer_as", cfg_key_nfprobe_peer_as},
+  {"sfacctd_time_new", cfg_key_nfacctd_time_new},
   {"sfacctd_pipe_size", cfg_key_nfacctd_pipe_size},
   {"sfacctd_renormalize", cfg_key_sfacctd_renormalize},
   {"sfacctd_disable_checks", cfg_key_nfacctd_disable_checks},
@@ -622,9 +625,21 @@ static const struct _dictionary_line dictionary[] = {
   {"sfacctd_counter_kafka_partition", cfg_key_sfacctd_counter_kafka_partition},
   {"sfacctd_counter_kafka_partition_key", cfg_key_sfacctd_counter_kafka_partition_key},
   {"sfacctd_counter_kafka_retry", cfg_key_sfacctd_counter_kafka_retry},
+  {"sfacctd_counter_kafka_config_file", cfg_key_sfacctd_counter_kafka_config_file},
   {"classifiers", cfg_key_classifiers},
   {"classifier_tentatives", cfg_key_classifier_tentatives},
   {"classifier_table_num", cfg_key_classifier_table_num},
+#if defined (WITH_NDPI)
+  {"classifier_num_roots", cfg_key_classifier_ndpi_num_roots},
+  {"classifier_max_flows", cfg_key_classifier_ndpi_max_flows},
+  {"classifier_proto_guess", cfg_key_classifier_ndpi_proto_guess},
+  {"classifier_idle_scan_period", cfg_key_classifier_ndpi_idle_scan_period},
+  {"classifier_idle_max_time", cfg_key_classifier_ndpi_idle_max_time},
+  {"classifier_idle_scan_budget", cfg_key_classifier_ndpi_idle_scan_budget},
+  {"classifier_giveup_proto_tcp", cfg_key_classifier_ndpi_giveup_proto_tcp},
+  {"classifier_giveup_proto_udp", cfg_key_classifier_ndpi_giveup_proto_udp},
+  {"classifier_giveup_proto_other", cfg_key_classifier_ndpi_giveup_proto_other},
+#endif
   {"nfprobe_timeouts", cfg_key_nfprobe_timeouts},
   {"nfprobe_hoplimit", cfg_key_nfprobe_hoplimit},
   {"nfprobe_maxflows", cfg_key_nfprobe_maxflows},
@@ -644,7 +659,6 @@ static const struct _dictionary_line dictionary[] = {
   {"sfprobe_direction", cfg_key_nfprobe_direction},
   {"sfprobe_ifindex", cfg_key_nfprobe_ifindex},
   {"sfprobe_ifspeed", cfg_key_sfprobe_ifspeed},
-  {"tee_receiver", cfg_key_nfprobe_receiver},
   {"tee_receivers", cfg_key_tee_receivers},
   {"tee_source_ip", cfg_key_nfprobe_source_ip},
   {"tee_transparent", cfg_key_tee_transparent},
@@ -652,9 +666,11 @@ static const struct _dictionary_line dictionary[] = {
   {"tee_max_receiver_pools", cfg_key_tee_max_receiver_pools},
   {"tee_ipprec", cfg_key_nfprobe_ip_precedence},
   {"tee_pipe_size", cfg_key_tee_pipe_size},
+  {"tee_dissect_send_full_pkt", cfg_key_tee_dissect_send_full_pkt},
   {"bgp_daemon", cfg_key_nfacctd_bgp},
   {"bgp_daemon_ip", cfg_key_nfacctd_bgp_ip},
   {"bgp_daemon_id", cfg_key_nfacctd_bgp_id},
+  {"bgp_daemon_as", cfg_key_nfacctd_bgp_as},
   {"bgp_daemon_port", cfg_key_nfacctd_bgp_port},
   {"bgp_daemon_pipe_size", cfg_key_nfacctd_bgp_pipe_size},
   {"bgp_daemon_max_peers", cfg_key_nfacctd_bgp_max_peers},
@@ -679,11 +695,15 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_daemon_msglog_kafka_partition", cfg_key_nfacctd_bgp_msglog_kafka_partition},
   {"bgp_daemon_msglog_kafka_partition_key", cfg_key_nfacctd_bgp_msglog_kafka_partition_key},
   {"bgp_daemon_msglog_kafka_retry", cfg_key_nfacctd_bgp_msglog_kafka_retry},
+  {"bgp_daemon_msglog_kafka_config_file", cfg_key_nfacctd_bgp_msglog_kafka_config_file},
   {"bgp_daemon_allow_file", cfg_key_nfacctd_bgp_allow_file},
   {"bgp_daemon_ipprec", cfg_key_nfacctd_bgp_ip_precedence},
   {"bgp_daemon_md5_file", cfg_key_nfacctd_bgp_md5_file},
   {"bgp_daemon_batch", cfg_key_nfacctd_bgp_batch},
   {"bgp_daemon_batch_interval", cfg_key_nfacctd_bgp_batch_interval},
+  {"bgp_daemon_offline_input", cfg_key_nfacctd_bgp_offline_input},
+  {"bgp_daemon_offline_file_spool", cfg_key_nfacctd_bgp_offline_file_spool},
+  {"bgp_daemon_offline_file_refresh_time", cfg_key_nfacctd_bgp_offline_file_refresh_time},
   {"bgp_aspath_radius", cfg_key_nfacctd_bgp_aspath_radius},
   {"bgp_stdcomm_pattern", cfg_key_nfacctd_bgp_stdcomm_pattern},
   {"bgp_extcomm_pattern", cfg_key_nfacctd_bgp_extcomm_pattern},
@@ -695,6 +715,7 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_peer_src_as_type", cfg_key_nfacctd_bgp_peer_src_as_type},
   {"bgp_src_std_comm_type", cfg_key_nfacctd_bgp_src_std_comm_type},
   {"bgp_src_ext_comm_type", cfg_key_nfacctd_bgp_src_ext_comm_type},
+  {"bgp_src_lrg_comm_type", cfg_key_nfacctd_bgp_src_lrg_comm_type},
   {"bgp_src_as_path_type", cfg_key_nfacctd_bgp_src_as_path_type},
   {"bgp_src_local_pref_type", cfg_key_nfacctd_bgp_src_local_pref_type},
   {"bgp_src_med_type", cfg_key_nfacctd_bgp_src_med_type},
@@ -728,6 +749,7 @@ static const struct _dictionary_line dictionary[] = {
   {"bgp_table_dump_kafka_topic_rr", cfg_key_nfacctd_bgp_table_dump_kafka_topic_rr},
   {"bgp_table_dump_kafka_partition", cfg_key_nfacctd_bgp_table_dump_kafka_partition},
   {"bgp_table_dump_kafka_partition_key", cfg_key_nfacctd_bgp_table_dump_kafka_partition_key},
+  {"bgp_table_dump_kafka_config_file", cfg_key_nfacctd_bgp_table_dump_kafka_config_file},
   {"bmp_daemon", cfg_key_nfacctd_bmp},
   {"bmp_daemon_ip", cfg_key_nfacctd_bmp_ip},
   {"bmp_daemon_port", cfg_key_nfacctd_bmp_port},
@@ -758,6 +780,7 @@ static const struct _dictionary_line dictionary[] = {
   {"bmp_daemon_msglog_kafka_partition", cfg_key_nfacctd_bmp_msglog_kafka_partition},
   {"bmp_daemon_msglog_kafka_partition_key", cfg_key_nfacctd_bmp_msglog_kafka_partition_key},
   {"bmp_daemon_msglog_kafka_retry", cfg_key_nfacctd_bmp_msglog_kafka_retry},
+  {"bmp_daemon_msglog_kafka_config_file", cfg_key_nfacctd_bmp_msglog_kafka_config_file},
   {"bmp_table_peer_buckets", cfg_key_nfacctd_bmp_table_peer_buckets},
   {"bmp_table_per_peer_buckets", cfg_key_nfacctd_bmp_table_per_peer_buckets},
   {"bmp_table_attr_hash_buckets", cfg_key_nfacctd_bmp_table_attr_hash_buckets},
@@ -783,6 +806,7 @@ static const struct _dictionary_line dictionary[] = {
   {"bmp_dump_kafka_topic_rr", cfg_key_nfacctd_bmp_dump_kafka_topic_rr},
   {"bmp_dump_kafka_partition", cfg_key_nfacctd_bmp_dump_kafka_partition},
   {"bmp_dump_kafka_partition_key", cfg_key_nfacctd_bmp_dump_kafka_partition_key},
+  {"bmp_dump_kafka_config_file", cfg_key_nfacctd_bmp_dump_kafka_config_file},
   {"flow_to_rd_map", cfg_key_nfacctd_flow_to_rd_map},
   {"isis_daemon", cfg_key_nfacctd_isis},
   {"isis_daemon_ip", cfg_key_nfacctd_isis_ip},
@@ -807,9 +831,7 @@ static const struct _dictionary_line dictionary[] = {
   {"uacctd_threshold", cfg_key_uacctd_threshold},
   {"tunnel_0", cfg_key_tunnel_0},
   {"pkt_len_distrib_bins", cfg_key_pkt_len_distrib_bins},
-  {"tmp_net_own_field", cfg_key_tmp_net_own_field},
   {"tmp_asa_bi_flow", cfg_key_tmp_asa_bi_flow},
-  {"tmp_comms_same_field", cfg_key_tmp_comms_same_field},
   {"", NULL},
 };
 
@@ -829,7 +851,8 @@ static struct plugin_type_entry plugin_types_list[] = {
   {PLUGIN_ID_SQLITE3,	"sqlite3",	sqlite3_plugin},
 #endif
 #ifdef WITH_MONGODB
-  {PLUGIN_ID_MONGODB,   "mongodb",      mongodb_plugin},
+  {PLUGIN_ID_UNKNOWN,	"mongodb",		mongodb_legacy_warning},
+  {PLUGIN_ID_MONGODB,  	"mongodb_legacy",	mongodb_plugin},
 #endif
 #ifdef WITH_RABBITMQ
   {PLUGIN_ID_AMQP,	"amqp",		amqp_plugin},
