@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -20,15 +20,15 @@
 */
 
 /* defines */
-#define ARGS_NFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:uRVaA:E:"
-#define ARGS_SFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:uRVaA:E:"
+#define ARGS_NFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:uRVaA:E:I:W"
+#define ARGS_SFACCTD "n:dDhP:b:f:F:c:m:p:r:s:S:L:l:o:t:O:uRVaA:E:I:W"
 #define ARGS_PMACCTD "n:NdDhP:b:f:F:c:i:I:m:p:r:s:S:o:t:O:uwWL:RVazA:E:"
 #define ARGS_UACCTD "n:NdDhP:b:f:F:c:m:p:r:s:S:o:t:O:uRg:L:VaA:E:"
-#define ARGS_PMTELEMETRYD "hVL:l:f:dDS:F:"
-#define ARGS_PMBGPD "hVL:l:f:dDS:F:"
-#define ARGS_PMBMPD "hVL:l:f:dDS:F:"
-#define ARGS_PMACCT "Ssc:Cetm:p:P:M:arN:n:lT:O:E:uDVUoiIx"
-#define N_PRIMITIVES 57
+#define ARGS_PMTELEMETRYD "hVL:l:f:dDS:F:o:O:i:"
+#define ARGS_PMBGPD "hVL:l:f:dDS:F:o:O:i:"
+#define ARGS_PMBMPD "hVL:l:f:dDS:F:o:O:i:"
+#define ARGS_PMACCT "Ssc:Cetm:p:P:M:arN:n:lT:O:E:uDVUiI"
+#define N_PRIMITIVES 75
 #define N_FUNCS 10 
 #define MAX_N_PLUGINS 32
 #define PROTO_LEN 12
@@ -39,12 +39,15 @@
 #define MAX_PROTOCOL_LEN 16
 #define MAX_PKT_LEN_DISTRIB_BINS 255
 #define MAX_PKT_LEN_DISTRIB_LEN 15
-#define DEFAULT_IMT_PLUGIN_SELECT_TIMEOUT 5
+#define DEFAULT_AVRO_SCHEMA_REFRESH_TIME 60
+#define DEFAULT_IMT_PLUGIN_POLL_TIMEOUT 5
 #define UINT32T_THRESHOLD 4290000000UL
 #define UINT64T_THRESHOLD 18446744073709551360ULL
 #define INT64T_THRESHOLD 9223372036854775807ULL
 #define PM_VARIABLE_LENGTH 65535
 #define PM_COUNTRY_T_STRLEN 4
+#define PM_POCODE_T_STRLEN 12
+#define PCAP_SAVEFILE_MAX_ERRORS 10
 #ifndef UINT8_MAX
 #define UINT8_MAX (255U)
 #endif
@@ -60,6 +63,9 @@
 #ifndef INT_MAX
 #define INT_MAX (2147483647U)
 #endif
+#ifndef INT_MIN
+#define INT_MIN (-2147483647 - 1)
+#endif
 
 #define LONGLONG_RETRY INT_MAX
 
@@ -71,6 +77,10 @@
 #define SNAPLEN_ISIS_MIN 512
 #define SNAPLEN_ISIS_DEFAULT 1476
 
+#define SUPERSHORTBUFLEN (16+MOREBUFSZ)
+#define VERYSHORTBUFLEN (32+MOREBUFSZ)
+#define SHORTSHORTBUFLEN (64+MOREBUFSZ)
+#define SHORTBUFLEN (128+MOREBUFSZ)
 #define SRVBUFLEN (256+MOREBUFSZ)
 #define LONGSRVBUFLEN (384+MOREBUFSZ)
 #define LONGLONGSRVBUFLEN (1024+MOREBUFSZ)
@@ -81,14 +91,15 @@
 #define PRIMITIVE_DESC_LEN	64
 
 #define MANTAINER "Paolo Lucente <paolo@pmacct.net>"
-#define PMACCTD_USAGE_HEADER "Promiscuous Mode Accounting Daemon, pmacctd 1.6.1"
-#define UACCTD_USAGE_HEADER "Linux NetFilter NFLOG Accounting Daemon, uacctd 1.6.1"
-#define PMACCT_USAGE_HEADER "pmacct, pmacct client 1.6.1"
-#define NFACCTD_USAGE_HEADER "NetFlow Accounting Daemon, nfacctd 1.6.1"
-#define SFACCTD_USAGE_HEADER "sFlow Accounting Daemon, sfacctd 1.6.1"
-#define PMTELEMETRYD_USAGE_HEADER "Streaming Network Telemetry Daemon, pmtelemetryd 1.6.1"
-#define PMBGPD_USAGE_HEADER "pmacct BGP Collector Daemon, pmbgpd 1.6.1"
-#define PMBMPD_USAGE_HEADER "pmacct BMP Collector Daemon, pmbmpd 1.6.1"
+#define GET_IN_TOUCH_MSG "If you see this message, please get in touch"
+#define PMACCTD_USAGE_HEADER "Promiscuous Mode Accounting Daemon, pmacctd 1.7.0"
+#define UACCTD_USAGE_HEADER "Linux NetFilter NFLOG Accounting Daemon, uacctd 1.7.0"
+#define PMACCT_USAGE_HEADER "pmacct, pmacct client 1.7.0"
+#define NFACCTD_USAGE_HEADER "NetFlow Accounting Daemon, nfacctd 1.7.0"
+#define SFACCTD_USAGE_HEADER "sFlow Accounting Daemon, sfacctd 1.7.0"
+#define PMTELEMETRYD_USAGE_HEADER "Streaming Network Telemetry Daemon, pmtelemetryd 1.7.0"
+#define PMBGPD_USAGE_HEADER "pmacct BGP Collector Daemon, pmbgpd 1.7.0"
+#define PMBMPD_USAGE_HEADER "pmacct BMP Collector Daemon, pmbmpd 1.7.0"
 #define PMACCT_COMPILE_ARGS COMPILE_ARGS
 #ifndef TRUE
 #define TRUE 1
@@ -97,7 +108,7 @@
 #define FALSE 0
 #endif
 #ifndef FALSE_NONZERO
-#define FALSE_NONZERO 2
+#define FALSE_NONZERO INT_MIN
 #endif
 #ifndef ERR
 #define ERR -1
@@ -145,7 +156,6 @@
 #define MAP_CUSTOM_PRIMITIVES	108	/* aggregate_primitives */
 
 /* PRIMITIVES DEFINITION: START */
-/* 55 primitives currently defined */
 /* internal: first registry, ie. what_to_count, aggregation, etc. */
 #define COUNT_INT_SRC_HOST		0x0001000000000001ULL
 #define COUNT_INT_DST_HOST		0x0001000000000002ULL
@@ -215,7 +225,16 @@
 #define COUNT_INT_LABEL			0x0002000000008000ULL
 #define COUNT_INT_EXPORT_PROTO_SEQNO	0x0002000000010000ULL
 #define COUNT_INT_EXPORT_PROTO_VERSION  0x0002000000020000ULL
-#define COUNT_INT_CUSTOM_PRIMITIVES	0x0002000000040000ULL
+#define COUNT_INT_LRG_COMM		0x0002000000040000ULL
+#define COUNT_INT_SRC_LRG_COMM		0x0002000000080000ULL
+#define COUNT_INT_SRC_HOST_POCODE	0x0002000000100000ULL
+#define COUNT_INT_DST_HOST_POCODE	0x0002000000200000ULL
+#define COUNT_INT_TUNNEL_SRC_HOST 	0x0002000000400000ULL
+#define COUNT_INT_TUNNEL_DST_HOST	0x0002000000800000ULL
+#define COUNT_INT_TUNNEL_IP_PROTO	0x0002000001000000ULL
+#define COUNT_INT_TUNNEL_IP_TOS		0x0002000002000000ULL
+#define COUNT_INT_NDPI_CLASS		0x0002000004000000ULL
+#define COUNT_INT_CUSTOM_PRIMITIVES	0x0002000008000000ULL
 
 #define COUNT_INDEX_MASK	0xFFFF
 #define COUNT_INDEX_CP		0xFFFF000000000000ULL  /* index 0xffff reserved to custom primitives */
@@ -291,6 +310,15 @@
 #define COUNT_LABEL			(COUNT_INT_LABEL & COUNT_REGISTRY_MASK)
 #define COUNT_EXPORT_PROTO_SEQNO	(COUNT_INT_EXPORT_PROTO_SEQNO & COUNT_REGISTRY_MASK)
 #define COUNT_EXPORT_PROTO_VERSION	(COUNT_INT_EXPORT_PROTO_VERSION & COUNT_REGISTRY_MASK)
+#define COUNT_LRG_COMM			(COUNT_INT_LRG_COMM & COUNT_REGISTRY_MASK)
+#define COUNT_SRC_LRG_COMM		(COUNT_INT_SRC_LRG_COMM & COUNT_REGISTRY_MASK)
+#define COUNT_SRC_HOST_POCODE		(COUNT_INT_SRC_HOST_POCODE & COUNT_REGISTRY_MASK)
+#define COUNT_DST_HOST_POCODE		(COUNT_INT_DST_HOST_POCODE & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_SRC_HOST		(COUNT_INT_TUNNEL_SRC_HOST & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_DST_HOST		(COUNT_INT_TUNNEL_DST_HOST & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_IP_PROTO		(COUNT_INT_TUNNEL_IP_PROTO & COUNT_REGISTRY_MASK)
+#define COUNT_TUNNEL_IP_TOS		(COUNT_INT_TUNNEL_IP_TOS & COUNT_REGISTRY_MASK)
+#define COUNT_NDPI_CLASS                (COUNT_INT_NDPI_CLASS & COUNT_REGISTRY_MASK)
 #define COUNT_CUSTOM_PRIMITIVES		(COUNT_INT_CUSTOM_PRIMITIVES & COUNT_REGISTRY_MASK)
 /* PRIMITIVES DEFINITION: END */
 
@@ -342,6 +370,7 @@
 #define PIPE_TYPE_MPLS		0x00000040
 #define PIPE_TYPE_VLEN		0x00000080
 #define PIPE_TYPE_LBGP		0x00000100
+#define PIPE_TYPE_TUN		0x00000200
 
 #define CHLD_WARNING		0x00000001
 #define CHLD_ALERT		0x00000002
@@ -389,6 +418,10 @@ typedef u_int32_t pm_class_t;
 typedef u_int64_t pm_id_t;
 typedef u_int64_t pm_cfgreg_t;
 
+#if defined (WITH_NDPI)
+typedef struct ndpi_proto pm_class2_t;
+#endif
+
 typedef struct {
   union {
     u_int32_t id;
@@ -396,6 +429,10 @@ typedef struct {
   };
 } pm_country_t;
 
+typedef struct {
+  char str[PM_POCODE_T_STRLEN];
+} pm_pocode_t;
+
 typedef struct {
   pm_cfgreg_t type;
   u_int32_t len;
