@@ -1,6 +1,6 @@
 /*  
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -45,6 +45,10 @@
 #include <malloc.h>
 #endif
 
+#if defined (HAVE_ZLIB)
+#include <zlib.h>
+#endif
+
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
@@ -52,6 +56,7 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <netinet/in.h>
+#include <net/if.h>
 #include <arpa/inet.h>
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -60,7 +65,9 @@
 #include <signal.h>
 #include <syslog.h>
 #include <sys/resource.h>
-#include <search.h>
+#include <dirent.h>
+#include <limits.h>
+#include "pmsearch.h"
 
 #include <sys/mman.h>
 #if !defined (MAP_ANONYMOUS)
@@ -82,6 +89,14 @@
 #include <maxminddb.h>
 #endif
 
+#if defined (WITH_NDPI)
+#include <ndpi_main.h>
+#endif
+
+#if defined (WITH_ZMQ)
+#include <zmq.h>
+#endif
+
 #include "pmacct-build.h"
 
 #if !defined ETHER_ADDRSTRLEN
@@ -204,6 +219,14 @@ typedef struct {
   u_int16_t off;
 } pm_hash_serial_t;
 
+#if (defined WITH_JANSSON)
+#include <jansson.h>
+#endif
+
+#if (defined WITH_AVRO)
+#include <avro.h>
+#endif
+
 #include "pmacct-defines.h"
 #include "network.h"
 #include "pretag.h"
@@ -236,6 +259,7 @@ struct pcap_device {
   pcap_t *dev_desc;
   int link_type;
   int active;
+  int errors; /* error count when reading from a savefile */
   struct _devices_struct *data; 
 };
 
@@ -269,7 +293,7 @@ struct _primitives_matrix_struct {
   u_int8_t sfacctd;
   u_int8_t pmtelemetryd;
   u_int8_t pmbgpd;
-  u_int8_t pmbgmd;
+  u_int8_t pmbmpd;
   char desc[PRIMITIVE_DESC_LEN];
 };
 
@@ -285,12 +309,6 @@ struct largebuf {
   u_char *ptr;
 };
 
-struct child_ctl {
-  u_int16_t active;
-  u_int16_t retired;
-  u_int32_t flags;
-};
-
 struct child_ctl2 {
   pid_t *list;
   u_int16_t active;
@@ -346,6 +364,7 @@ EXT void pcap_cb(u_char *, const struct pcap_pkthdr *, const u_char *);
 EXT int PM_find_id(struct id_table *, struct packet_ptrs *, pm_id_t *, pm_id_t *);
 EXT void compute_once();
 EXT void set_index_pkt_ptrs(struct packet_ptrs *);
+EXT ssize_t recvfrom_savefile(struct pcap_device *, void **, struct sockaddr *, struct timeval **);
 #undef EXT
 
 #ifndef HAVE_STRLCPY
@@ -353,12 +372,22 @@ size_t strlcpy(char *, const char *, size_t);
 #endif
 
 #if (defined WITH_JANSSON)
-#include <jansson.h>
 #if (!defined HAVE_JSON_OBJECT_UPDATE_MISSING)
 int json_object_update_missing(json_t *, json_t *);
 #endif
 #endif
 
+void
+#ifdef __STDC__
+pm_setproctitle(const char *fmt, ...);
+#else /* __STDC__ */
+#error
+pm_setproctitle(fmt, va_alist);
+#endif /* __STDC__ */
+
+void
+initsetproctitle(int, char**, char**);
+
 /* global variables */
 #if (!defined __PMACCTD_C) && (!defined __NFACCTD_C) && (!defined __SFACCTD_C) && (!defined __UACCTD_C) && (!defined __PMTELEMETRYD_C) && (!defined __PMBGPD_C) && (!defined __PMBMPD_C)
 #define EXT extern
@@ -371,7 +400,6 @@ EXT int reload_map_bgp_thread, reload_log_bgp_thread, reload_log_bmp_thread;
 EXT int reload_log_sf_cnt, reload_log_telemetry_thread;
 EXT int data_plugins, tee_plugins;
 EXT struct timeval reload_map_tstamp;
-EXT struct child_ctl sql_writers;
 EXT struct child_ctl2 dump_writers;
 EXT int debug;
 EXT struct configuration config; /* global configuration structure */
