@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -27,6 +27,7 @@ const struct _map_dictionary_line tag_map_dictionary[] = {
   {"out", PT_map_output_handler},
   {"engine_type", PT_map_engine_type_handler},
   {"engine_id", PT_map_engine_id_handler},
+  {"source_id", PT_map_engine_id_handler},
   {"nexthop", PT_map_nexthop_handler},
   {"bgp_nexthop", PT_map_bgp_nexthop_handler},
   {"filter", PT_map_filter_handler},
@@ -57,6 +58,7 @@ const struct _map_dictionary_line tag_map_dictionary[] = {
   {"jeq", PT_map_jeq_handler},
   {"return", PT_map_return_handler},
   {"stack", PT_map_stack_handler},
+  {"fwdstatus", PT_map_fwdstatus_handler},
   {"", NULL}
 };
 
@@ -70,11 +72,13 @@ const struct _map_index_dictionary_line tag_map_index_entries_dictionary[] = {
   {PRETAG_PEER_SRC_AS, PT_map_index_entries_peer_src_as_handler},
   {PRETAG_PEER_DST_AS, PT_map_index_entries_peer_dst_as_handler},
   {PRETAG_MPLS_LABEL_BOTTOM, PT_map_index_entries_mpls_label_bottom_handler},
+  {PRETAG_MPLS_VPN_ID, PT_map_index_entries_mpls_vpn_id_handler},
   {PRETAG_MPLS_VPN_RD, PT_map_index_entries_mpls_vpn_rd_handler},
   {PRETAG_SRC_MAC, PT_map_index_entries_src_mac_handler},
   {PRETAG_DST_MAC, PT_map_index_entries_dst_mac_handler},
   {PRETAG_VLAN_ID, PT_map_index_entries_vlan_id_handler},
   {PRETAG_CVLAN_ID, PT_map_index_entries_cvlan_id_handler},
+  {PRETAG_FWDSTATUS_ID, PT_map_index_entries_fwdstatus_handler},
   {0, NULL}
 };
 
@@ -88,11 +92,13 @@ const struct _map_index_dictionary_line tag_map_index_fdata_dictionary[] = {
   {PRETAG_PEER_SRC_AS, PT_map_index_fdata_peer_src_as_handler},
   {PRETAG_PEER_DST_AS, PT_map_index_fdata_peer_dst_as_handler},
   {PRETAG_MPLS_LABEL_BOTTOM, PT_map_index_fdata_mpls_label_bottom_handler},
+  {PRETAG_MPLS_VPN_ID, PT_map_index_entries_mpls_vpn_id_handler},
   {PRETAG_MPLS_VPN_RD, PT_map_index_fdata_mpls_vpn_rd_handler},
   {PRETAG_SRC_MAC, PT_map_index_fdata_src_mac_handler},
   {PRETAG_DST_MAC, PT_map_index_fdata_dst_mac_handler},
   {PRETAG_VLAN_ID, PT_map_index_fdata_vlan_id_handler},
   {PRETAG_CVLAN_ID, PT_map_index_fdata_cvlan_id_handler},
+  {PRETAG_FWDSTATUS_ID, PT_map_index_fdata_fwdstatus_handler},
   {0, NULL}
 };
 
@@ -106,6 +112,11 @@ const struct _map_dictionary_line tag_map_tee_dictionary[] = {
   {"out", PT_map_output_handler},
   {"src_mac", PT_map_src_mac_handler},
   {"dst_mac", PT_map_dst_mac_handler},
+  {"vlan", PT_map_vlan_id_handler},
+  {"engine_type", PT_map_engine_type_handler},
+  {"engine_id", PT_map_engine_id_handler},
+  {"source_id", PT_map_engine_id_handler},
+  {"agent_id", PT_map_agent_id_handler},
   {"label", PT_map_entry_label_handler},
   {"jeq", PT_map_jeq_handler},
   {"return", PT_map_return_handler},
@@ -150,6 +161,7 @@ const struct _map_dictionary_line bitr_map_dictionary[] = {
   {"in", PT_map_input_handler},
   {"out", PT_map_output_handler},
   {"bgp_nexthop", PT_map_bgp_nexthop_handler},
+  {"mpls_vpn_id", BITR_map_mpls_vpn_id_handler}, 
   {"mpls_label_bottom", BITR_map_mpls_label_bottom_handler},
   {"", NULL}
 };
