@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -24,6 +24,7 @@
 
 /* includes */
 #include "pmacct.h"
+#include "addr.h"
 #include "nfacctd.h"
 #include "pretag_handlers.h"
 #include "pretag-data.h"
@@ -32,6 +33,7 @@
 #include "isis/isis.h"
 #include "isis/isis-data.h"
 #include "crc32.h"
+#include "pmacct-data.h"
 
 /*
    XXX: load_id_file() interface cleanup pending:
@@ -349,8 +351,35 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	        if (tmp.e[tmp.num].id && tmp.e[tmp.num].id2 && tmp.e[tmp.num].label.len) 
 		   Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] set_tag (id), set_tag2 (id2) and set_label are mutual exclusive. Line ignored.\n", 
 			config.name, config.type, filename, tot_lines);
-                else if (!err && tmp.e[tmp.num].key.agent_ip.a.family) {
-                  int j, z;
+                else if (!err) {
+                  int j, z, recirculate;
+		  struct id_entry recirc_e;
+
+		  recirculate = FALSE;
+		  recirculate_ipv6:
+
+		  if (!tmp.e[tmp.num].key.agent_ip.a.family) {
+		    if (!recirculate) {
+		      memset(&tmp.e[tmp.num].key.agent_ip, 0, sizeof(pt_hostaddr_t));
+		      memset(&tmp.e[tmp.num].key.agent_mask, 0, sizeof(pt_hostmask_t));
+		      tmp.e[tmp.num].key.agent_ip.a.family = AF_INET;
+		      tmp.e[tmp.num].key.agent_mask.family = AF_INET;
+
+#if defined ENABLE_IPV6
+		      memcpy(&recirc_e, &tmp.e[tmp.num], sizeof(struct id_entry));
+		      recirculate = TRUE;
+#endif
+		    }
+#if defined ENABLE_IPV6
+		    else {
+		      memcpy(&tmp.e[tmp.num], &recirc_e, sizeof(struct id_entry));
+                      tmp.e[tmp.num].key.agent_ip.a.family = AF_INET6;
+                      tmp.e[tmp.num].key.agent_mask.family = AF_INET6;
+
+		      recirculate = FALSE;
+		    }
+#endif
+		  }
 
                   for (j = 0; tmp.e[tmp.num].func[j]; j++);
 		  for (z = 0; tmp.e[tmp.num].set_func[z]; z++, j++) {
@@ -363,12 +392,13 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	          else if (tmp.e[tmp.num].key.agent_ip.a.family == AF_INET6) v6_num++;
 #endif
                   tmp.num++;
+
+		  if (recirculate) {
+		    if (tmp.num < map_entries) goto recirculate_ipv6;
+		  }
                 }
 	        /* if any required field is missing and other errors have been signalled
 	           before we will trap an error message */
-	        else if (!err && !tmp.e[tmp.num].key.agent_ip.a.family)
-	          Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] required key missing. Required key is: 'ip'. Line ignored.\n",
-			config.name, config.type, filename, tot_lines); 
 	      }
 	      else if (acct_type == ACCT_PM) {
 	        if (tmp.e[tmp.num].id && tmp.e[tmp.num].id2 && tmp.e[tmp.num].label.len)
@@ -407,7 +437,7 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 			config.name, config.type, filename, tot_lines);
 	      }
               else if (acct_type == MAP_BGP_TO_XFLOW_AGENT) {
-                if (!err && tmp.e[tmp.num].id && tmp.e[tmp.num].key.agent_ip.a.family) {
+                if (!err && (tmp.e[tmp.num].id || tmp.e[tmp.num].id2) && tmp.e[tmp.num].key.agent_ip.a.family) {
                   int j, z;
 
                   for (j = 0; tmp.e[tmp.num].func[j]; j++);
@@ -424,7 +454,7 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 #endif
                   tmp.num++;
                 }
-                else if ((!tmp.e[tmp.num].id || !tmp.e[tmp.num].key.agent_ip.a.family) && !err)
+                else if (((!tmp.e[tmp.num].id && !tmp.e[tmp.num].id2) || !tmp.e[tmp.num].key.agent_ip.a.family) && !err)
                   Log(LOG_WARNING, "WARN ( %s/%s ): [%s:%u] required key missing. Required keys are: 'id', 'ip'. Line ignored.\n",
                         config.name, config.type, filename, tot_lines);
               }
@@ -567,12 +597,11 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
 	t->index_num = MAX_ID_TABLE_INDEXES;
 
 #if defined ENABLE_IPV6
-        for (ptr = t->ipv4_base, x = 0; x < MAX(t->ipv4_num, t->ipv6_num); ptr++, x++) {
+        for (ptr = t->ipv4_base, x = 0; x < (t->ipv4_num + t->ipv6_num); ptr++, x++) {
 #else
         for (ptr = t->ipv4_base, x = 0; x < t->ipv4_num; ptr++, x++) {
 #endif
 	  idx_bmap = pretag_index_build_bitmap(ptr, acct_type);
-	  if (!idx_bmap) continue;
 
 	  /* insert bitmap to index list and determine entries per index */ 
 	  if (pretag_index_insert_bitmap(t, idx_bmap)) {
@@ -590,12 +619,11 @@ void load_id_file(int acct_type, char *filename, struct id_table *t, struct plug
         pretag_index_allocate(t);
 
 #if defined ENABLE_IPV6
-        for (ptr = t->ipv4_base, x = 0; x < MAX(t->ipv4_num, t->ipv6_num); ptr++, x++) {
+        for (ptr = t->ipv4_base, x = 0; x < (t->ipv4_num + t->ipv6_num); ptr++, x++) {
 #else
         for (ptr = t->ipv4_base, x = 0; x < t->ipv4_num; ptr++, x++) {
 #endif
           idx_bmap = pretag_index_build_bitmap(ptr, acct_type);
-          if (!idx_bmap) continue;
 
 	  /* fill indexes */
 	  pretag_index_fill(t, idx_bmap, ptr);
@@ -845,11 +873,6 @@ pt_bitmap_t pretag_index_build_bitmap(struct id_entry *ptr, int acct_type)
   if (idx_bmap & PRETAG_SET_TAG2) idx_bmap ^= PRETAG_SET_TAG2;
   if (idx_bmap & PRETAG_SET_LABEL) idx_bmap ^= PRETAG_SET_LABEL;
 
-  /* 3) add 'ip' to bitmap, if mandated by the map type */
-  if (acct_type == ACCT_NF || acct_type == ACCT_SF ||
-      acct_type == MAP_BGP_PEER_AS_SRC || acct_type == MAP_FLOW_TO_RD)
-    idx_bmap |= PRETAG_IP;
-
   return idx_bmap;
 }
 
@@ -860,7 +883,7 @@ int pretag_index_insert_bitmap(struct id_table *t, pt_bitmap_t idx_bmap)
   if (!t) return TRUE;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
-    if (!t->index[iterator].bitmap || t->index[iterator].bitmap == idx_bmap) {
+    if (!t->index[iterator].entries || t->index[iterator].bitmap == idx_bmap) {
       t->index[iterator].bitmap = idx_bmap;
       t->index[iterator].entries++;
       return FALSE;
@@ -922,7 +945,7 @@ int pretag_index_allocate(struct id_table *t)
   if (!t) return TRUE;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
-    if (t->index[iterator].bitmap) {
+    if (t->index[iterator].entries) {
       Log(LOG_INFO, "INFO ( %s/%s ): [%s] maps_index: created index %x (%u entries).\n", config.name,
     		config.type, t->filename, t->index[iterator].bitmap, t->index[iterator].entries);
 
@@ -971,7 +994,7 @@ int pretag_index_fill(struct id_table *t, pt_bitmap_t idx_bmap, struct id_entry
   if (!t) return ERR;
 
   for (iterator = 0; iterator < t->index_num; iterator++) {
-    if (t->index[iterator].bitmap && t->index[iterator].bitmap == idx_bmap) {
+    if (t->index[iterator].entries && t->index[iterator].bitmap == idx_bmap) {
       struct id_entry e;
       struct id_index_entry *idie;
       pm_hash_serial_t *hash_serializer;
