@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -29,39 +29,41 @@
 #define ID_TABLE_INDEX_DEPTH 8
 #define ID_TABLE_INDEX_RESULTS (MAX_ID_TABLE_INDEXES * 8)
 
-#define PRETAG_IN_IFACE			0x000000001
-#define PRETAG_OUT_IFACE		0x000000002
-#define PRETAG_NEXTHOP			0x000000004
-#define PRETAG_BGP_NEXTHOP		0x000000008
-#define PRETAG_ENGINE_TYPE		0x000000010
-#define PRETAG_ENGINE_ID		0x000000020
-#define PRETAG_FILTER			0x000000040
-#define PRETAG_NFV8_AGG			0x000000080
-#define PRETAG_SF_AGENTID		0x000000100
-#define PRETAG_SAMPLING_RATE		0x000000200
-#define PRETAG_DIRECTION		0x000000400
-#define PRETAG_SRC_AS			0x000000800
-#define PRETAG_DST_AS			0x000001000
-#define PRETAG_PEER_SRC_AS		0x000002000
-#define PRETAG_PEER_DST_AS		0x000004000
-#define PRETAG_SRC_LOCAL_PREF		0x000008000
-#define PRETAG_LOCAL_PREF		0x000010000
-#define PRETAG_SRC_STD_COMM		0x000020000
-#define PRETAG_STD_COMM			0x000040000
-#define PRETAG_MPLS_VPN_RD		0x000080000
-#define PRETAG_SAMPLE_TYPE      	0x000100000
-#define PRETAG_SET_TOS			0x000200000
-#define PRETAG_LOOKUP_BGP_PORT		0x000400000
-#define PRETAG_SET_TAG			0x000800000
-#define PRETAG_SET_TAG2			0x001000000
-#define PRETAG_MPLS_LABEL_BOTTOM	0x002000000
-#define PRETAG_FLOWSET_ID		0x004000000
-#define PRETAG_SRC_MAC			0x008000000
-#define PRETAG_DST_MAC			0x010000000
-#define PRETAG_VLAN_ID			0x020000000
-#define PRETAG_IP			0x040000000
-#define PRETAG_SET_LABEL		0x080000000
-#define PRETAG_CVLAN_ID			0x100000000
+#define PRETAG_IN_IFACE			0x000000001ULL
+#define PRETAG_OUT_IFACE		0x000000002ULL
+#define PRETAG_NEXTHOP			0x000000004ULL
+#define PRETAG_BGP_NEXTHOP		0x000000008ULL
+#define PRETAG_ENGINE_TYPE		0x000000010ULL
+#define PRETAG_ENGINE_ID		0x000000020ULL
+#define PRETAG_FILTER			0x000000040ULL
+#define PRETAG_NFV8_AGG			0x000000080ULL
+#define PRETAG_SF_AGENTID		0x000000100ULL
+#define PRETAG_SAMPLING_RATE		0x000000200ULL
+#define PRETAG_DIRECTION		0x000000400ULL
+#define PRETAG_SRC_AS			0x000000800ULL
+#define PRETAG_DST_AS			0x000001000ULL
+#define PRETAG_PEER_SRC_AS		0x000002000ULL
+#define PRETAG_PEER_DST_AS		0x000004000ULL
+#define PRETAG_SRC_LOCAL_PREF		0x000008000ULL
+#define PRETAG_LOCAL_PREF		0x000010000ULL
+#define PRETAG_SRC_STD_COMM		0x000020000ULL
+#define PRETAG_STD_COMM			0x000040000ULL
+#define PRETAG_MPLS_VPN_RD		0x000080000ULL
+#define PRETAG_SAMPLE_TYPE      	0x000100000ULL
+#define PRETAG_SET_TOS			0x000200000ULL
+#define PRETAG_LOOKUP_BGP_PORT		0x000400000ULL
+#define PRETAG_SET_TAG			0x000800000ULL
+#define PRETAG_SET_TAG2			0x001000000ULL
+#define PRETAG_MPLS_LABEL_BOTTOM	0x002000000ULL
+#define PRETAG_FLOWSET_ID		0x004000000ULL
+#define PRETAG_SRC_MAC			0x008000000ULL
+#define PRETAG_DST_MAC			0x010000000ULL
+#define PRETAG_VLAN_ID			0x020000000ULL
+#define PRETAG_IP			0x040000000ULL
+#define PRETAG_SET_LABEL		0x080000000ULL
+#define PRETAG_CVLAN_ID			0x100000000ULL
+#define PRETAG_MPLS_VPN_ID		0x200000000ULL
+#define PRETAG_FWDSTATUS_ID		0x400000000ULL
 
 #define PRETAG_MAP_RCODE_ID		0x00000100
 #define PRETAG_MAP_RCODE_ID2		0x00000200
@@ -105,7 +107,7 @@ typedef struct host_mask pt_hostmask_t;
 
 typedef struct {
   u_int8_t neg;
-  u_char a[ETH_ADDR_LEN]; 
+  char a[ETH_ADDR_LEN]; 
 } pt_etheraddr_t;
 
 typedef struct {
@@ -144,7 +146,7 @@ struct id_entry_key {
   pt_uint32_t input; /* input interface index */
   pt_uint32_t output; /* output interface index */
   pt_uint8_t engine_type;
-  pt_uint8_t engine_id;
+  pt_uint32_t engine_id;
   pt_uint16_t flowset_id; /* applies to NetFlow v9/IPFIX flowset ID */
   pt_uint32_t agent_id; /* applies to sFlow agentSubId */
   pt_uint32_t sampling_rate; /* applies to sFlow sampling rate */
@@ -164,7 +166,9 @@ struct id_entry_key {
   s_uint16_t lookup_bgp_port;
   char *src_comms[16]; /* XXX: MAX_BGP_COMM_PATTERNS = 16 */
   char *comms[16]; /* XXX: MAX_BGP_COMM_PATTERNS = 16 */
+  pt_uint32_t mpls_vpn_id;
   pt_rd_t mpls_vpn_rd;
+  pt_uint32_t fwdstatus;
   struct bpf_program filter;
   pt_uint8_t v8agg;
 };
