@@ -383,10 +383,22 @@ inline static void putClass(SFLReceiver *receiver, SFLExtended_classification *c
   put128(receiver, buf);
 }
 
+inline static void putClass2(SFLReceiver *receiver, SFLExtended_classification2 *class2_elem)
+{
+  u_int64_t dummy = 0;
+
+#if defined (WITH_NDPI)
+  putNet32(receiver, class2_elem->id.master_protocol);
+  putNet32(receiver, class2_elem->id.app_protocol);
+#else
+  putNet64(receiver, dummy);
+#endif
+}
+
 inline static void putTag(SFLReceiver *receiver, SFLExtended_tag *tag_elem)
 {
-  putNet32(receiver, tag_elem->tag);
-  putNet32(receiver, tag_elem->tag2);
+  putNet64(receiver, tag_elem->tag);
+  putNet64(receiver, tag_elem->tag2);
 }
 
 inline static void putGenericCounters(SFLReceiver *receiver, SFLIf_counters *counters)
@@ -456,7 +468,8 @@ static int computeFlowSampleSize(SFLReceiver *receiver, SFL_FLOW_SAMPLE_TYPE *fs
     case SFLFLOW_EX_MPLS_LDP_FEC: elemSiz = mplsLdpFecEncodingLength(&elem->flowType.mpls_ldp_fec); break;
     case SFLFLOW_EX_VLAN_TUNNEL: elemSiz = vlanTunnelEncodingLength(&elem->flowType.vlan_tunnel); break;
     case SFLFLOW_EX_CLASS: elemSiz = MAX_PROTOCOL_LEN; break;
-    case SFLFLOW_EX_TAG: elemSiz = 8; break;
+    case SFLFLOW_EX_CLASS2: elemSiz = 8; break;
+    case SFLFLOW_EX_TAG: elemSiz = 16; /* (sizeof(pm_id_t) * 2) */ break;
     default:
       sflError(receiver, "unexpected packet_data_tag");
       return -1;
@@ -584,6 +597,7 @@ int sfl_receiver_writeFlowSample(SFLReceiver *receiver, SFL_FLOW_SAMPLE_TYPE *fs
     case SFLFLOW_EX_MPLS_LDP_FEC: putMplsLdpFec(receiver, &elem->flowType.mpls_ldp_fec); break;
     case SFLFLOW_EX_VLAN_TUNNEL: putVlanTunnel(receiver, &elem->flowType.vlan_tunnel); break;
     case SFLFLOW_EX_CLASS: putClass(receiver, &elem->flowType.class); break;
+    case SFLFLOW_EX_CLASS2: putClass2(receiver, &elem->flowType.ndpi_class); break;
     case SFLFLOW_EX_TAG: putTag(receiver, &elem->flowType.tag); break;
     default:
       sflError(receiver, "unexpected packet_data_tag");
