@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2016 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2017 by Paolo Lucente
 */
 
 /*
@@ -35,6 +35,9 @@
 #include "sql_common.h"
 #include "ip_flow.h"
 #include "classifier.h"
+#if defined (WITH_NDPI)
+#include "ndpi/ndpi.h"
+#endif
 
 static const char fake_mac[] = "0:0:0:0:0:0";
 static const char fake_host[] = "0.0.0.0";
@@ -221,6 +224,22 @@ void count_dst_host_country_handler(const struct db_cache *cache_elem, struct in
   *ptr_where += strlen(*ptr_where);
   *ptr_values += strlen(*ptr_values);
 }
+
+void count_src_host_pocode_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->primitives.src_ip_pocode.str);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->primitives.src_ip_pocode.str);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void count_dst_host_pocode_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->primitives.dst_ip_pocode.str);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->primitives.dst_ip_pocode.str);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
 #endif
 
 void count_sampling_rate_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
@@ -309,6 +328,61 @@ void count_mpls_stack_depth_handler(const struct db_cache *cache_elem, struct in
   *ptr_values += strlen(*ptr_values);
 }
 
+void count_tunnel_src_ip_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char ptr[INET6_ADDRSTRLEN];
+
+  addr_to_str(ptr, &cache_elem->ptun->tunnel_src_ip);
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ptr);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ptr);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void count_tunnel_dst_ip_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char ptr[INET6_ADDRSTRLEN];
+
+  addr_to_str(ptr, &cache_elem->ptun->tunnel_dst_ip);
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ptr);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ptr);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void MY_count_tunnel_ip_proto_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  if (cache_elem->ptun->tunnel_proto < protocols_number) {
+    snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, _protocols[cache_elem->ptun->tunnel_proto].name);
+    snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, _protocols[cache_elem->ptun->tunnel_proto].name);
+  }
+  else {
+    char proto_str[PROTO_LEN];
+
+    snprintf(proto_str, sizeof(proto_str), "%d", cache_elem->ptun->tunnel_proto);
+    snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, proto_str);
+    snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, proto_str);
+  }
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void PG_count_tunnel_ip_proto_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->ptun->tunnel_proto);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->ptun->tunnel_proto);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void count_tunnel_ip_tos_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, cache_elem->ptun->tunnel_tos);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, cache_elem->ptun->tunnel_tos);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
 void PG_copy_count_timestamp_start_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
 {
   static char time_str[LONGSRVBUFLEN];
@@ -531,6 +605,19 @@ void count_ext_comm_handler(const struct db_cache *cache_elem, struct insert_dat
   *ptr_values += strlen(*ptr_values);
 }
 
+void count_lrg_comm_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char *bgp_comm_ptr = NULL, empty_string[] = "";
+
+  vlen_prims_get(cache_elem->pvlen, COUNT_INT_LRG_COMM, &bgp_comm_ptr);
+  if (!bgp_comm_ptr) bgp_comm_ptr = empty_string;
+
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, bgp_comm_ptr);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, bgp_comm_ptr);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
 void count_as_path_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
 {
   char *as_path_ptr = NULL, empty_string[] = "";
@@ -570,6 +657,19 @@ void count_src_ext_comm_handler(const struct db_cache *cache_elem, struct insert
   *ptr_values += strlen(*ptr_values);
 }
 
+void count_src_lrg_comm_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char *bgp_comm_ptr = NULL, empty_string[] = "";
+
+  vlen_prims_get(cache_elem->pvlen, COUNT_INT_SRC_LRG_COMM, &bgp_comm_ptr);
+  if (!bgp_comm_ptr) bgp_comm_ptr = empty_string;
+
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, bgp_comm_ptr);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, bgp_comm_ptr);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
 void count_src_as_path_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
 {
   char *as_path_ptr = NULL, empty_string[] = "";
@@ -695,7 +795,6 @@ void count_ip_tos_handler(const struct db_cache *cache_elem, struct insert_data
   *ptr_values += strlen(*ptr_values);
 }
 
-
 void MY_count_ip_proto_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
 {
   if (cache_elem->primitives.proto < protocols_number) {
@@ -794,6 +893,22 @@ void count_class_id_handler(const struct db_cache *cache_elem, struct insert_dat
   *ptr_values += strlen(*ptr_values);
 }
 
+#if defined (WITH_NDPI)
+void count_ndpi_class_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char ndpi_class[SUPERSHORTBUFLEN];
+
+  snprintf(ndpi_class, SUPERSHORTBUFLEN, "%s/%s",
+	ndpi_get_proto_name(pm_ndpi_wfl->ndpi_struct, cache_elem->primitives.ndpi_class.master_protocol),
+	ndpi_get_proto_name(pm_ndpi_wfl->ndpi_struct, cache_elem->primitives.ndpi_class.app_protocol));
+
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, ndpi_class);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, ndpi_class);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+#endif
+
 void count_counters_setclause_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_set, char **ptr_none)
 {
   snprintf(*ptr_set, SPACELEFT(set_clause), set[num].string, cache_elem->packet_counter, cache_elem->bytes_counter);
@@ -871,11 +986,11 @@ void count_src_host_aton_handler(const struct db_cache *cache_elem, struct inser
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->primitives.src_ip);
-  if (cache_elem->primitives.src_ip.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->primitives.src_ip.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
   
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -889,11 +1004,11 @@ void count_dst_host_aton_handler(const struct db_cache *cache_elem, struct inser
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->primitives.dst_ip);
-  if (cache_elem->primitives.dst_ip.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->primitives.dst_ip.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -907,11 +1022,11 @@ void count_src_net_aton_handler(const struct db_cache *cache_elem, struct insert
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->primitives.src_net);
-  if (cache_elem->primitives.src_net.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->primitives.src_net.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -925,11 +1040,11 @@ void count_dst_net_aton_handler(const struct db_cache *cache_elem, struct insert
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->primitives.dst_net);
-  if (cache_elem->primitives.dst_net.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->primitives.dst_net.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -943,11 +1058,11 @@ void count_peer_src_ip_aton_handler(const struct db_cache *cache_elem, struct in
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->pbgp->peer_src_ip);
-  if (cache_elem->pbgp->peer_src_ip.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->pbgp->peer_src_ip.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -961,11 +1076,11 @@ void count_peer_dst_ip_aton_handler(const struct db_cache *cache_elem, struct in
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->pbgp->peer_dst_ip);
-  if (cache_elem->pbgp->peer_dst_ip.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->pbgp->peer_dst_ip.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -979,11 +1094,11 @@ void count_post_nat_src_ip_aton_handler(const struct db_cache *cache_elem, struc
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->pnat->post_nat_src_ip);
-  if (cache_elem->pnat->post_nat_src_ip.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->pnat->post_nat_src_ip.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
@@ -997,11 +1112,47 @@ void count_post_nat_dst_ip_aton_handler(const struct db_cache *cache_elem, struc
   char ptr[INET6_ADDRSTRLEN];
 
   addr_to_str(ptr, &cache_elem->pnat->post_nat_dst_ip);
-  if (cache_elem->pnat->post_nat_dst_ip.family == AF_INET) aton = aton_v4;
 #if defined ENABLE_IPV6
-  else if (cache_elem->pnat->post_nat_dst_ip.family == AF_INET6) aton = aton_v6;
+  aton = aton_v6;
+#else
+  aton = aton_v4;
+#endif
+
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void count_tunnel_src_ip_aton_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char *aton = NULL, aton_v4[] = "INET_ATON", aton_v6[] = "INET6_ATON", aton_null[] = " ";
+  char ptr[INET6_ADDRSTRLEN];
+
+  addr_to_str(ptr, &cache_elem->ptun->tunnel_src_ip);
+#if defined ENABLE_IPV6
+  aton = aton_v6;
+#else
+  aton = aton_v4;
+#endif
+
+  snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
+  snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
+  *ptr_where += strlen(*ptr_where);
+  *ptr_values += strlen(*ptr_values);
+}
+
+void count_tunnel_dst_ip_aton_handler(const struct db_cache *cache_elem, struct insert_data *idata, int num, char **ptr_values, char **ptr_where)
+{
+  char *aton = NULL, aton_v4[] = "INET_ATON", aton_v6[] = "INET6_ATON", aton_null[] = " ";
+  char ptr[INET6_ADDRSTRLEN];
+
+  addr_to_str(ptr, &cache_elem->ptun->tunnel_dst_ip);
+#if defined ENABLE_IPV6
+  aton = aton_v6;
+#else
+  aton = aton_v4;
 #endif
-  else aton = aton_null;
 
   snprintf(*ptr_where, SPACELEFT(where_clause), where[num].string, aton, ptr);
   snprintf(*ptr_values, SPACELEFT(values_clause), values[num].string, aton, ptr);
